// Board specific definitions for JUMA-TRX2 transceiver
// Juha Niinikoski OH2NLT 22.11.2006
/*
 MPLAB C30 version
 
 S-meter default calibration changed for Rev.B board. 2.0V = full scale. 17.03.2007
 Voice memory Rev.B board definition changes 21.03.2007
 Button switch de-bounce time shortened 03.04.2007
 All Band PA filter select 15.04.2007
 PTT_IN changed to PTT_IN_IO for main logic change 02.06.2007
 CW TX side tone SCAF setup added 25.01.2008
 K250 & K600 filter switch points changed 18.04.2009
 DEFAULT_TX filter changed to 2,6kHz (2586Hz), 19.04.2009
*/
// Processor header
#include <p30f6014A.h>

// Version Data
#define VERSION					"5.00a"
#define BUILD					"8"
#define BUILD_DATE				"04/JUL/2019"

#define	INCLUDE_TEST_CODE		FALSE
#define MEASURE_CYCLE_TIME		FALSE

/*
 This defines a structure identified as: DDS_Registers containing an object of type
 unsigned long long called: Frequency. (Used in ad9850.c and serial_test.c)
*/
struct DDS_Registers			// Synthesiser frequency data
	{
	unsigned long long Frequency;
	};

// Macro Definitions
#define lcd_spc(count)			set_ch_bits(' ', count)			// Print spaces
#define start_scaf_clock()		T2CONbits.TON = 1				// Start TMR2
#define stop_scaf_clock()		T2CONbits.TON = 0				// Stop TMR2
#define set_pwm3_dac(pwm)		OC3RS = pwm						// LCD Backlight
#define set_pwm4_dac(pwm)		OC4RS = pwm						// LCD Contrast
#define tone_off()				last_cycle = 2					// Stop tone generator
#define bl_off()				set_pwm3_dac(0)					// Turn backlight off
#define display_page			eeprom.defval.mode[39]			// Spare location, now used as the display page
#define power_down_mode			eeprom.defval.nb				// Shorthand for the power-down mode descriptor
#define hold_latch				cal.calval.button_mode			// Shorthand for the page hold/latch function
#define clear_lcd()				lcd_cmd(CLEAR)
#define maximum_VFO				eeprom.defval.max_vfo			// Shorthand for the maximum number of VFOs
#define Select_VFO				eeprom.defval.vfo_sel			// Shorthand for current VFO operating mode, Simplex VFO-A/B, or Split
#define Equalisation_Mode		eeprom.defval.VFO_Equalisation	// Shorthand for VFO Equalisation Mode, Off, Split + Simplex, Split Only
#define AGC_speed				eeprom.defval.agc_speed			// Shorthand for AGC speed (Fast/Slow)
#define Speech_Processor		eeprom.defval.proc_on_off		// Shorthand for Speech Processor switch (On/Off)
#define AF_Input				eeprom.defval.af_input			// Shorthand for audio input source (Mic/Line)
#define Tuning_Step				eeprom.defval.step_idx			// Shorthand for tuning rate (1Hz/10Hz/100Hz/1kHz)
#define RIT_On_Off				eeprom.defval.rit_on			// Shorthand for Receiver Incremental Tuning (On/Off)
#define Serial_Mode				eeprom.defval.serial_mode		// Shorthand for RS-232 Port Mode (Juma-TRX2/Yaesu CAT/Kenwood CAT/Test/Voice Memory)
#define Display_Scale			eeprom.defval.Display_Type		// Shorthand for graphic display scale type (Original/Large Markers/Small Markers)
#define RBS_Mode				eeprom.defval.Band_SW			// Shorthand for Rapid Band-Switch mode (Default/Mode-A/Mode-B/Locked/Mode-C)
#define RX_Bandwidth			eeprom.defval.filter			// Shorthand for receiver bandwidth (Narrow/Wide/Transmit)
#define Keyer_Mode				eeprom.defval.keyer				// Shorthand for autokeyer mode (Dot Priority/Iambic A/Iambic B/Vibroplex/Straight)
#define Baud_Rate				eeprom.defval.br				// Shorthand for RS-232 port speed
#define Keyer_Speed				eeprom.defval.CW_Speed			// Shorthand for the CW Keyer speed
/*
 From an examination of the code, it was clear that the button de-bounce and the primary/secondary button function selection logic could be combined.
 In this version, the function exits if button_count is either zero, or if it is greater than the de-bounce limit. In the first case this would occur
 if the button was held for a long time, in other words, a secondary function selection. In the second case, this would occur if the button was only
 pressed for a short time. In the main software loop, button_count is set to the value DEBOUNCE_COUNT each time the loop cycles. DEBOUNCE_LIMIT is
 set to a value somewhat longer than DEBOUNCE_COUNT. The variable button_count is a long integer. A.Ryan - 5B4AIY - 29/DEC/2016
*/
#define button_debounce(button)	do {if(button) button_count--; else button_count += 2;} while(button_count && (button_count < DEBOUNCE_LIMIT))
/*
 Initial Code Size							: 77,577
 The code: button_count++ 					: 77,586, +11 bytes, 4 instructions, Execution Time = 0.5333uS
 With code: if(_MODE) button_count++;		: 77,607, +30 bytes, 10 instructions, Execution Time = 1.3333uS
 With addition code: else button_count--;	: 77,619, +12 bytes, 4 instructions, Execution Time = 0.5333uS
 With DO loop code							: 77,640, +21 bytes, 7 instructions, Execution Time = 0.9333uS

 Thus, in order to have a long push time of around 300mS, the code: do{if(_MODE) button_count++} while(button_count && (button_count < DEBOUNCE_LIMIT));
 is executed and this would take the DO Loop - 0.9333uS, IF(_MODE) delay_time++; - 1.3333uS, making 2.2667uS.
 For a total long push delay of 300mS, this would require at least 132,350 iterations.
 Since the up-count is only really required to remove contact bounce, a time of 15mS is more than long enough, and thus an additional 6,617 is sufficient.
 Thus, DEBOUNCE_COUNT is set to 125,000, and DEBOUNCE_LIMIT is set to 130,000
 Using a long integer, button_count for the de-bounce count, the code size is: 77,661 bytes.

 With this implementation, when a button push is registered, button_count will tend to oscillate around its initial setting until the contact is fully
 closed or fully open. In the first case this will cause button_count to decrement to zero. In the second, button_count will initially decrement
 until the button is released completely, whereupon it will increment until the limit is reached. In the first case, if button_count is zero, this
 indicates a long push, and a secondary function selection, whereas if button_count contains a value, (in this case DEBOUNCE_LIMIT), then this is
 a short push, a primary function selection.

 One minor 'quirk' of this straightforward method, and that is if the button is held for just less that the long push time, then there is a very noticeable
 delay before the short push function executes. To obviate this, the macro is modified so that the incrementing occurs twice as fast, and to compensate,
 the DEBOUNCE_LIMIT value is increased. With this modification, there is almost no observable delay when releasing the button for the primary function
 to be executed, but button contact bounces are still suppressed. A.Ryan - 5B4AIY - 09/JAN/2017
*/
// General Definitions
#define TRUE					1
#define FALSE					0
#define ON						TRUE
#define OFF						FALSE
#define ENABLE					TRUE
#define DISABLE					FALSE
#define YES						TRUE
#define NO						FALSE
#define NUL						0
#define FAST					1
#define	SLOW					0
#define NARROW					0						// [N] bandwidth RX Filter
#define	WIDE					1						// [W] bandwidth RX filter
#define	TX_FILTER				2						// TX Filter
#define SPARE					3						// Spare filter bandwidth (Unused)
#define LEFT_ARROW				0x7F					// Left direction LCD arrow
#define RIGHT_ARROW				0x7E					// Right direction LCD arrow
#define LOCKED					0x8000					// CW Speed locking
#define UNLOCKED				0x7FFF					// CW Speed unlocking
#define	TRANSMIT				0
#define RECEIVE					1
#define AUDIO_ON				0
#define AUDIO_OFF				1
#define IARU_1					0
#define USA_EXTRA				1
#define USA_GENERAL				2
#define JUMA					1
#define JUMA_REGION				2
#define HOLD					0						// RBS/VFO Copy/VFO/Select/FPS mode. Press & Hold
#define LATCH					1						// RBS/VFO Copy/VFO/Select/FPS mode. Latched with beep
#define NORMAL					0						// VFO knob rotation sense. C/W to increase
#define REVERSE					1						// VFO knob rotation sense. ACW to increase
#define SPACE					' '
#define ZERO					'0'
#define PLUS_ONE				+1
#define MINUS_ONE				-1

// Auto-Equalise Constants
#define	NORMAL_EXIT				0
#define ABORT					TRUE
#define	PWR_MEASUREMENT_ERR		1
#define	USER_ABORT				2
#define	ALARM_ERR				3
#define MIN_POWER				800						// Minimum Auto Equalise Power, mW

// Tuning
#define	RF_FILT_TABLE_LEN		11

// Display Control
#define NORMAL_DISPLAY			0						// Normal Display Mode
#define CONFIG_DISPLAY			1						// User Configuration Menu Display
#define ALARM_DISPLAY			2
#define MAX_LCD_MODE			1						// Last display "main" page
#define	MAX_SUB_PAGE0			4						// Sub page 0 displays
#define	MAX_SUB_PAGE1			30						// Number of User Configuration pages
#define	LAST_PAGE				MAX_SUB_PAGE0 + 1
#define	MODE_BW					0
#define PA_CURRENT				4
#define PWR_MTR_DECAY			70

// UART setup according clock config
#define DEFAULT_BAUD_RATE		3						// Default baud rate = 9600
#define	DISABLED				0						// RS-232 port disabled
#define SERIAL_YAESU			1						// Yaesu CAT emulation
#define SERIAL_KENWOOD			2						// Kenwood CAT emulation
#define SERIAL_JUMA_TRX2		3						// Juma TRX2-PA100D Mode
#define SERIAL_TEST 			4						// Test Mode
#define SERIAL_VOICE_MEMORY		5						// Voice Memory mode

// Clock constants
#define FCY						7500000UL				// 30MHz ext Osc / 4  = 7,5MHz
#define SCAF_CLK				FCY / 100UL				// Switched Capacitor Audio Filter Clock Frequency
#define DLYCONST				((FCY / 1000000UL) + 1)	// Constant for timer delay routines

// TRX-2 common definitions, used by various modules
// DDS constants
#define REF_OSC					180000000				// 180 MHz reference oscillator nominal frequency
#define MAX_OSC					180020000
#define MIN_OSC					179980000
#define MAX_DDS					30000000				// Normal Upper Frequency
#define DEFAULT_FREQ_A			3600000					// "Factory Default" VFO A 3.6 MHz
#define DEFAULT_FREQ_B			7060000					// "Factory Default" VFO B 7.060 MHz
#define MIN_DDS					0						// Low limit of 0 allows DDS test at audio frequencies
#define M7_FILT					4000000					// 3,5 / 7MHz (Two Band PA) filter switch at 4MHz

// Service Mode
#define SERVICE_FREQ			14019300				// Standard Service & Calibration Frequency (USB) using Elecraft XG3 signal generator

// All-Band PA filter & attenuator switch points
#define K300					300000					// 300kHz, 18.04.2009 
#define K600					600000					// 600kHz

#define M1						1000000					// 1.000000 MHz
#define M2						2001000					// 2.001000 MHz
#define M4						4001000					// 4.001000 MHz
#define M8						8000000					// 8.000000 MHz
#define M12						12000000				// 12.000000 MHz
#define M15						15000000				// 15.000000 MHz
#define M19						19000000				// 19.000000 MHz
#define M23						23000000				// 23.000000 MHz
#define M26						26000000				// 26.000000 MHz

// Attenuator Limit Values
#define MAX_ATTEN				3
#define MIN_ATTEN				0

#define CW_SHIFT 				700						// Default CW shift.
#define CW_SIDETONE				5357					// Default tone 700 Hz (7500000 / (2 * Tone)
#define MIN_CW_SHIFT			300						// 300 Hz
#define MAX_CW_SHIFT			1100					// 1100Hz
#define CB_SUPPRESSION_TIME		10						// CW Keyer Straight Mode contact bounce suppression time (mS)

// RIT adjustment
#define RIT_SCALE				24L						// RIT pot scaling factor (Not used in decimal frequency generator system) - 5B4AIY
#define RIT_DEAD_ZONE			47						// RIT pot center dead zone, A/D steps - New dead zone for decimal frequency system - 5B4AIY

// CW Keyer
#define KEY_DOWN				TRUE
#define KEY_UP					FALSE
#define DOT_PRIORITY			0
#define IAMBIC_A				1
#define IAMBIC_B				2						// Factory default = Iambic B
#define VIBROPLEX				3
#define STRAIGHT_KEY			4
#define DEFAULT_CW_RELEASE		500						// Default CW Straight Key Release, 500mS
#define MIN_CW_RELEASE			190						// Minimum CW Straight Key release time 200mS, 190mS indicates normal mode 
#define MAX_CW_RELEASE			1500					// Maximum CW Straight Key release time 1,500mS

// LCD parameters & factory defaults
#define DEFAULT_CONTRAST		2000
#define MAX_CONTRAST			3500
#define MIN_CONTRAST			50
#define DEFAULT_BL				300
#define MAX_BL					1000
#define MIN_BL					50
#define DEFAULT_BLT				0						// LCD back light timer not in use
#define MAX_BLT					720						// 1 hour (5-Second increments)
#define MIN_S_METER_HOLD		250						// Minimum Numeric S-Meter Hold Time, 250mS
#define MAX_S_METER_HOLD		5000					// Maximum Numeric S-Meter Hold Time, 5000mS

// Display Commands
#define CURSET					0x80					// Set memory cursor
#define	CLEAR					0x01					// Clear display
#define ADRSET					0x40					// Set CGRAM address

// 2 line x 16 character LCD specific definitions
// LCD Memory Layout
#define LN1						0x00					// Address of Line 1
#define LN2						0x40					// Address of Line 2
#define LINE1					LN1 | CURSET			// Address 0x00 | CURSET
#define LINE2					LN2 | CURSET			// Address 0x40 | CURSET
#define MODE_FIELD				(LN1 + 9) | CURSET		// Address 0x09 | CURSET
#define LINE1PLUS4				(LN1 + 4) | CURSET		// Address 0x04 | CURSET
#define LINE1PLUS7				(LN1 + 7) | CURSET		// Address 0x07 | CURSET
#define LINE2PLUS1				(LN2 + 1) | CURSET		// Address 0x41 | CURSET
#define LINE2PLUS4				(LN2 + 4) | CURSET		// Address 0x45 | CURSET
#define LINE2PLUS15				(LN2 + 15) | CURSET		// Address 0x4F | CURSET

// Graphic Meter Scale Constants
#define DISP_LEN				8						// Display length (characters)
#define FONT_W					6						// Font width (pixels)
#define MAX_GRAPH				(DISP_LEN * FONT_W)		// Graph length in pixels (5*16 = 80)
#define END_OF_SCALE			0x07					// Meter End-Of-Scale symbol
#define FULL_BLOCK				0x05
/*
 When using the TRX-Manager CAT program, the software requires a scaled value of the current output power.
 The maximum value is 48, corresponding to the 8 display characters with 6 segments used as a graphic
 bar-meter. The RF detector has an approximate scale factor of 2.05V for a 10W output. If the A-D converter
 reference voltage is exactly 5.000V, then this would lead to a raw A-D value of:

	4096 * 2.05 / 5.0 = 1679

 When measuring power, this has to be squared, and then multiplied by the power meter calibration factor,
 giving the output power in nW, thus, the default calibration constant is:

	10,000,000,000 / ( 1679 * 1679 ) = 3547

 This would give a power reading of:

	1679 * 1679 * 3547 = 9,999,138,427nW

 To scale this value to 48 requires a division constant of:

	9,999,138,427 / 48 = 208315383

 In the actual scaling operation, the forward power is multiplied by the calibration constant divided by
 this constant to give a multiplier of 0.0000170271.
 The multiplier has to take into consideration the current power meter calibration constant, and thus it
 is calculated once during the boot sequence.
*/
#define PWR_BAR_CONSTANT		208315383.0

// Mode Change Blink Rate Definitions
#define MODE_CHANGE				80						// Governs the speed of mode/filter change blink
#define MODE_COUNTER			(4 * MODE_CHANGE) - 1

// Panel Buttons & Timers
#define DEBOUNCE_COUNT			125000					// Used in macro: button_debounce(button)
#define DEBOUNCE_LIMIT			135000
#define BUTTON_DEBOUNCE			105						// De-bounce delay(ms) for fast fingers (OH7SV)
#define DEFAULT_BEEP_LEN		50						// Button beep length 50ms, long = * 10
#define BEEP_LEN_MAX			100						// Maximum value of beep time 100mS
#define BEEP_LEN_MIN			0						// Minimum value of beep time (OFF)
#define SHORT_BEEP				50						// Forced short beep
#define LONG_BEEP				500						// Forced long beep
#define MS_TIMER				999						// For 1 second timers in timers_pwm.c
#define GLITCH_TIME				5						// 5mS for glitch suppression in exchg_data_spi1()

#define SQ_HOLD					850						// Squelch hold time, 850mS
#define ENCODER_SLOW			50
#define ENCODER_MEDIUM			25
#define ENCODER_NORMAL			5

// Tuning Rates
#define ONE_HZ					0
#define TEN_HZ					1
#define HUNDRED_HZ				2
#define KILO_HZ					3
#define TEN_KILO_HZ				4

// ADC Channel Definitions
#define ID_CUR					9						// PA Drain current channel
#define BATT_CH					10						// Battery channel
#define S_METER					11						// S-Meter
#define REV_PWR					12						// Reverse power channel
#define FWD_PWR					13						// Forward power
#define CW_POT					14						// CW speed pot, R24
#define RIT_POT					15						// RIT adjust pot, R9

// Tone generator constants, 7,5MHz / tone(Hz) * 2
// Some fixed tones
#define HZ350					10714					// 350Hz
#define HZ400					9375					// 400Hz
#define HZ500					7500					// 500Hz
#define HZ700					5357					// 700Hz
#define	HZ800					4687					// 800Hz
#define HZ1000					3750					// 1 kHz
#define HZ1500					2500					// 1.5kHz
#define HZ2000					1875					// 2kHz

// Some harmony sounds
#define HZ392_01				9566					// G, off (392.01Hz)
#define HZ466_85				8044					// B-flat, push button standard tone (466.185Hz)
#define HZ587_31				6385					// D, fast tune tone (587.31Hz)
#define HZ698_45				5369					// F, CW side tone (698.45Hz)

// Meter scaling factors, factory defaults
#define BATT_MULT				5300					// V = (ADC * BATT_MULT) / 1000000
#define BATT_MULT_MAX			6000					// Maximum multiplier value
#define BATT_MULT_MIN			4000					// Minimum multiplier value
#define ID_MULT					2400					// I = (ADC * ID_MULT) / 2000000
#define ID_MULT_MAX				3600					// Maximum multiplier value - 5B4AIY
#define ID_MULT_MIN				1200					// Minimum multiplier value - 5B4AIY
#define	S_METER_CAL				20						// Default value for S-Meter scaling, S-meter = ADC / S_MTR_CAL, 48 = max meter = 2.0V FS
#define S_METER_CAL_MAX 		70						// Maximum S-Meter calibration value
#define S_METER_CAL_MIN			5						// Minimum S-Meter calibration value
#define	FWD_PWR_MULT			3550					// PnW = (ADC * ADC) * FWD_PWR_MULT / 1000000000
#define PWR_MTR_DEAD_BAND		500						// Approximately 888mW
#define FWD_PWR_MULT_MAX		5000					// Maximum multiplier
#define FWD_PWR_MULT_MIN		1500					// Minimum multiplier

// Alarm Bits
/*
 The alarm bits are mapped as:
 D0	High Current
 D1	High SWR
 D2	High Voltage
 D3	Low Voltage (Final Limit)
 D4	Low Voltage (Pre-Limit)
 D5 - D15 Unused

 The EEPROM value: cal.calval.alarm_flags is used as the alarm mask.
*/
#define OC_AL_ON				0x01
#define OC_AL_OFF				0x0E
#define CLR_OC_AL				0x1E					// This preserves a possible Low Voltage pre-limit alarm
#define SWR_AL					0x02
#define CLR_SWR_AL				0x1D					// This preserves a possible Low Voltage pre-limit alarm
#define HI_V_AL_ON				0x04
#define HI_V_AL_OFF				0x0B
#define CLR_HI_V_AL				0x1B					// This preserves the pre-limit alarm in the Serial Test Suite
#define LO_V_AL_ON				0x08
#define LO_V_AL_OFF				0x07
#define CLR_LO_V_AL				0x17					// This preserves a possible Low Voltage pre-limit alarm
#define BATT_LO					0x10
#define CLR_BATT_LO				0x0F					// This clears the pre-limit alarm, but preserves a Low Voltage alarm

// Alarm limits
#define SWR_ALARM				300						// SWR alarm 3:1
#define MAX_RO					0.818182				// See notes in juma-trx2.c, calc_swr() function.
#define MAX_SWR_ALARM			1000					// SWR alarm limit 10.0:1
#define MIN_SWR_ALARM			100						// Minimum SWR alrm 1:1
#define SWR_AL_DLY				500						// PTT to alarm active delay 500ms
#define MIN_SWR_DLY				0						// The minimum delay is 1 Second, but this value is used to determine the disabled state.
#define MAX_SWR_DLY				10						// 10,0 Sec
#define LONG_SWR_DLY			5						// Default Long SWR Alarm Delay, 5 seconds
#define OVERVOLTAGE_UPPER		1500					// Maximum over-voltage trip = 15.01V
#define OVERVOLTAGE_LOWER		1400					// Minimum over-voltage trip = 14.01V
#define OVERVOLTAGE_TRIP		2736					// Nominal over-voltage trip = 14.5V
#define UNDERVOLTAGE_UPPER		1150					// Maximum under-voltage trip = 11.5V
#define UNDERVOLTAGE_LOWER		1050					// Minimum under-voltage trip = 10.5V
#define UNDERVOLTAGE_TRIP		2076					// Nominal under-voltage trip = 11.0V
#define OVERCURRENT_UPPER		3500					// Maximum over-current trip = 3.5A - for Tim O'Rourke
#define OVERCURRENT_LOWER		1500					// Minimum over-current trip = 1.5A
#define OVERCURRENT_TRIP		2084					// Nominal over-current trip = 2.5A
#define OVERCURRENT_AVERAGE		64						// Number of counts to average the current
#define LOOP_COUNT				400						// Alarm blink loop counter count-down value (400mS decremented in timers_pwm.c, 1mSec tick timer)
#define PEAK_SHOW_TIME			1000					// Display time for peak power meter (1S)

// SCAF filter clock
// Divider = 7500000 / Filt(Hz) * 100
#define DEFAULT_NAR				75						// 1kHz   (1,000Hz)
#define DEFAULT_WID				30						// 2,5kHz (2,500Hz)
#define DEFAULT_TX				29						// 2,6kHz (2,586Hz), 19.04.2009

#define SIDE_TONE_SCAF			50						// Side-tone filter for CW TX
#define MID_SCAF				30						// Set value wide upper limit, 2,500Hz
#define NAR_SCAF				58						// Set value narrow upper and wide lower limit, 1,293Hz
#define MAX_SCAF				150						// Set value high limit, 500Hz
#define TX_LOWER				33						// Minimum bandwidth 2,273Hz 
#define TX_UPPER				20						// Maximum bandwidth 3,750Hz

// Power-On & Calibration value addresses in the EEPROM
#define EEPAGE					0x7F					// EEPROM address high part
#define EEDEF					0xF000					// EEPROM default values storage area
#define EECAL					0xF150					// EEPROM calibration values storage area
#define EE_FD_LOC				0xF1FE					// Counter address in EEPROM

// Microphone / Line input
#define MIC						1						// Microphone input level / Bias current ON
#define LINE					0						// Line input level / Bias OFF 

// TRX2 board I/O definitions
// Port A Switches
#define INIT_TRISA				0xFF3F					// Port-A switch inputs
#define INIT_PORTA				0x0000

#define SW2						_RA12					// FAST/VFAST, 0 = Pressed
#define SW1						_RA13					// DISPLAY/CONFIG, 0 = Pressed
#define ENQ_B					_RA14					// Rotary encoder, Phase B / INT 3
#define ENQ_A					_RA15					// Rotary encoder, Phase A / INT 4

// Port B ADC Inputs
#define INIT_TRISB				0xFE73					// Port-B ADC and IDC pins = Inputs
#define INIT_PORTB				0x0000

#define PTT_IN_IO				_RB6					// PTT input / TX ON, 0 = Active
#define PTT_OUT					_LATB7					// PTT output, 1 = Active

#define DOT						_RB5					// Paddle DOT input, 0 = Active
#define DASH					_RB4					// Paddle DASH input, 0 = Active
#define KEY						_LATB3					// Key output, 1 = Active

// Port C Switches & Power Control
#define INIT_TRISC				0x401E
#define INIT_PORTC				0x0000

#define SW6						_RC1					// FILTER, 0 = Pressed
#define SW5						_RC2					// MODE, 0 = Pressed
#define SW4						_RC3					// VFO, 0 = Pressed
#define SW3						_RC4					// RIT, 0 = Pressed

#define PWR_SW					_RC14					// PWR, 1 = Pressed
#define PWR_ON					_RC13					// Power ON

// Front Panel Buttons
#define _FILTER					!_RC1					// FILTER button pressed
#define _MODE					!_RC2					// MODE button pressed
#define _VFO					!_RC3					// VFO button pressed
#define _RIT					!_RC4					// RIT button pressed
#define _VFAST					!_RA12					// FAST/VFAST button pressed
#define _DISPLAY				!_RA13					// DISPLAY button pressed

// Port D LCD & PWM out
#define INIT_TRISD 				0x0000					// LCD & LED I/O, all outputs
#define INIT_PORTD				0x0000

// Tone output
#define TONE_OUT				_LATD1					// Tone output
#define TONE_TRIS				_TRISD1					// Tone output tri-state control

// LCD signals
#define LCD_E					_LATD5					// LCD E signal
#define LCD_RW					_LATD6					// LCD R/W signal
#define LCD_RS					_LATD7					// LCD RS signal

#define LCD_DATA				LATD					// LCD 8-bit data bus out register
#define LCD_BUS					PORTD					// LCD 8-bit data bus state
#define LCD_TRIS				TRISD					// LCD data direction 

#define LCD_SHIFT 				8						// LCD bits shifted from D7/D0 to D15/D8
#define LCD_MASK				0xFF00					// Used to extract the 8-bit LCD data read in from the 16-bit I/O port.
#define LCD_BUSY_FLAG			0x8000					// Used to extract the BUSY flag from the 16-bit I/O port data.

// PORT F & G 
// RS-232, DDS, SPI and Codec/AUX signals
//#define INIT_TRISF 			0xFC7C
#define INIT_TRISF 				0xFE18
#define INIT_PORTF 				0x0000

#define INIT_TRISG 				0x1080					// I/O directions for ISD17240 add-on board
#define INIT_PORTG 				0x0000

#define DDS_SER					_LATF0					// AD9850 DDS D7/SER
#define DDS_RESET				_LATG0					// AD9850 DDS Reset
#define DDS_FQ_UD				_LATG1					// AD9850 DDS FQ_UD
#define DDS_W_CLK				_LATF1					// AD9850 DDS W_CLK

// 74HC595 Main board shift register bits
// Shift register is loaded with SPI1 functions, MSB goes first
#define SSB_CW					(1 << 0)				// SSB / !CW Select 1 = USB/LSB, 0 = CW/CWR/Tune
#define SB_SELECT				(1 << 1)				// Sideband Select, 1 = LSB/CW/Tune, 0 = USB/CWR
#define NAR						(1 << 2)				// Narrow BPF Filter, 1 = Narrow
#define FAST_AGC				(1 << 3)				// Fast AGC Select, 1 = Fast
#define PROC_ON					(1 << 4)				// TX Speech Processor, 1 = ON
#define NB						(1 << 5)				// Noise Blanker - No longer used - 5B4AIY
#define	AF_INPUT				(1 << 6)				// Microphone / Line Input Select, 0 = Line, 1 = MIC
#define MHZ7					(1 << 7)				// 3,5 / 7 MHz Filter Change

// ISD17240 Voice Recorder I/O
#define ISD_17240				0xE0					// ISD17240 Device ID
#define ISD_PLAY_BAND 			_LATG15					// Out - Play to Band switch, 1 = Play Band
#define ISD_RESET				_LATG14					// Out - ISD17240 nRESET
#define ISD_RDY					_RG12					// In  - ISD17240 Ready / INT
#define ISD_SS					_LATG9					// Out - ISD17240 SPI nSS
#define ISD_SCLK				_LATG6					// Out - ISD17240 SPI SCLK
#define ISD_MOSI				_LATG8					// Out - ISD17240 SPI MOSI
#define ISD_MISO				_RG7					// In  - ISD17240 SPI MISO

// Voice Memory Mode Indicator
#define RECORD					-1						// Decrement seconds counter of Voice Memory module
#define REPLAY					+1						// Increment seconds counter of Voice Memory module
#define IDLE					0

// Amateur Band Centre Frequencies & Band Edges
#define _160M_CENTRE			1900000L				// Centre frequency of 160m band
#define _160M_UPPER				2000000L				// Upper band edge of 160m band
#define _160M_LOWER				1810000L				// Lower band edge of 160m band

#define _80M_CENTRE				3600000L				// Default frequency for 80m (Legal for US General)
#define _80M_UPPER_USA			4000000L				// US Upper Band Edge
#define _80M_UPPER_IARU			3800000L				// Upper band edge of 80m band
#define _80M_LOWER				3500000L				// Lower band edge of 80m band

#define _40M_CENTRE				7100000L				// Centre frequency of 40m band
#define _40M_UPPER_USA			7300000L				// US Upper Band Edge
#define _40M_UPPER_IARU			7200000L				// Upper band edge of 40m band
#define _40M_LOWER				7000000L				// Lower band edge of 40m band

#define _30M_CENTRE				10125000L				// Centre frequency of 30m band
#define _30M_UPPER				10150000L				// Upper band edge of 30m band
#define _30M_LOWER				10100000L				// Lower band edge of 30m band

//#define _20M_CENTRE			14175000L				// Centre frequency of 20m band
#define _20M_CENTRE				14100000L				// Default frequency for 20m (Legal for US General)
#define _20M_UPPER				14350000L				// Upper band edge of 20m band
#define _20M_LOWER				14000000L				// Lower band edge of 20m band

#define _17M_CENTRE				18118000L				// Centre frequency of 17m band
#define	_17M_UPPER				18168000L				// Upper band edge of 17m band
#define _17M_LOWER				18068000L				// Lower band edge of 17m band

#define _15M_CENTRE				21300000L				// Default frequency for 15m (Legal for US General)
#define _15M_UPPER				21450000L				// Upper band edge of 15m band
#define	_15M_LOWER				21000000L				// Lower band edge of 15m band

#define _12M_CENTRE				24940000L				// Centre frequency of 12m band
#define _12M_UPPER				24990000L				// Upper band edge of 12m band
#define _12M_LOWER				24890000L				// Lower band edge of 12m band

#define _10M_CENTRE				28850000L				// Centre frequency of 10m band
#define _10M_UPPER				29700000L				// Upper band edge of 10m band
#define _10M_LOWER				28000000L				// Lower band edge of 10m band
/*
  The following frequencies are used in the Rapid Bandswitch routine to select the nearest amateur band
  compared to the current frequency. If the current VFO frequency < selected frequency, then the lower
  amateur band is selected.
*/
#define	SELECT_160				2750001L				// 2.75Mhz
#define	SELECT_80				5400001L				// 5.4MHz
#define	SELECT_40				8650001L				// 8.65MHz
#define	SELECT_30				12075001L				// 12.075MHz
#define	SELECT_20				16209001L				// 16.209MHz
#define	SELECT_17				19584001L				// 19.584MHz
#define	SELECT_15				23170001L				// 23.17MHz
#define	SELECT_12				26495001L				// 26.495MHz

// Mode Definitions
#define	LSB						0
#define	USB						1
#define	CW						2
#define	CWR						3
#define	TUNE					4
#define VFO_A					0
#define VFO_B					VFO_A + 1
#define VFO_Z					VFO_A + 25
#define SPLIT_MODE				2
#define SPLIT_ONLY				1
#define SPLIT_PLUS_SIMPLEX		2

// Power Off Definitions
#define IMMEDIATE				0
#define PROMPTED				1
#define SAVE_STATE				2

// Rapid Bandswitch Mode Definitions
#define _DEFAULT				0
#define _MODE_A					1
#define _MODE_B					2
#define _LOCKED					3
#define OUT_OF_BAND				0
#define LOWEST_BAND				1
#define HIGHEST_BAND			9

// Graphic Scale Definitions
#define ORIGINAL_MARKERS		0
#define LARGE_MARKERS			1
#define SMALL_MARKERS			2

// S-Meter Scale Definitions
#define GRAPHIC					0
#define NUMERIC					1

// Meter Function Definitions
#define WATTS					0
#define	SWR						1
#define	VOLTS					2
#define	AMPS					3
#define DBM						4

// Frequency Display Definitions
#define	ORIGINAL				0
#define NEW_DISP				1
#define FIXED_A					2
#define FIXED_B					3

// CW Annunciation Constants
#define LTR_F					0b10000100
#define LTR_S					0b01100000
#define LTR_V					0b10001000

// CW Annunciation Status
#define SKIP_CYCLE				0x0010
#define FULL_STATUS				SKIP_CYCLE + 0x000F
#define MODE_STATUS				SKIP_CYCLE + 0x0001
#define FILTER_STATUS			SKIP_CYCLE + 0x0002
#define VFO_STATUS				SKIP_CYCLE + 0x0004
#define FREQ_STATUS				SKIP_CYCLE + 0x0008

