/*
 JUMA-TRX2 serial I/O Yaesu CAT interface simulator
 Juha Niinikoski, OH2NLT

 Test version 12.01.2007
 VFO lock added 18.01.2007
 Real timer for protocol time-out & different logic for CAT CMD0x01  22.01.2008
 Set frequency vfo_idx pointer use corrected 31.01.2008
 Set Mode vfo_idx pointer use corrected 09.02.2008
 Yaesu CAT Read TX status command 0xF7 bug corrected, squelch bit added to 0xE7 response 28.01.2009
 Undocumented feature discovered from Yaesu 897 CAT protocol
 0xE7 response is 0xFF when TX and 0xF7 response is 0xFF when RX 29.01.2009
 0x00 & 0xF0 & 0xFF responses added 29.01.2009 
 Dirty patch for undocumented Yaesu commands 00 xx 00 00 BB to get HRD working 30.01.2009
 Added Yaesu Command Description - Adrian Ryan - 5B4AIY
 Modified to use decimal frequencies in Hz now that synthesiser calculations are done in decimal - 5B4AIY
 Modified to take account of CWR mode - 5B4AIY
 Added bcd2bin() and bin2bcd() functions. Replaced in-line code for frequency conversion with more
 compact code in the convert_cat_freq() function and the serial_yaesu() functions. This saved almost 600
 bytes of code. Adrian Ryan - 5B4AIY - 25/JUN/12


 Yaesu Command Description

 Yaesu CAT commands are organised as a 5-byte sequence, with the parameters first, and the last byte being the
 command byte.

 The following commands are emulated:

 Read RX frequency & mode	: 0x03
 Read RX status				: 0xE7
 Read TX status				: 0xF7
 Split Mode ON				: 0x02
 Split Mode OFF				: 0x82
 VFO Select					: 0x81
 Set operating mode			: 0x07
 Set frequency				: 0x01
 PTT ON (TX)				: 0x08
 PTT OFF (RX)				: 0x88
 Lock VFO					: 0x00
 Unlock VFO					: 0x80
 Toggle VFO-A/VFO-B			: 0x81
 Clarifier On (RIT)			: 0x05
 Clarifier Off (RIT)		: 0x85
 Set Tuning Rate			: 0xE0	Note: Not part of standard Yaesu protocol!
 Set Receiver Bandwidth		: 0xE8	Note: Not part of standard Yaesu protocol!
 Extended Display			: 0xF8	Note: Not part of standard Yaesu protocol!

 Command Format
 The commands take the form of a 5-byte string terminating with the command byte. The leading 4 bytes are the
 parameter values required. If a command does not require any parameters, then these bytes can contain any
 value, but nulls are preferred.

 Read RX frequency and mode (Assume the transceiver's frequency is 145.43210Mhz
 Command To Transceiver	: 00 00 00 00 03
 Transceiver Response	: 14 54 32 10 xx

 xx is the mode byte, with the following meaning: 00 = LSB, 01 = USB, 02 = CW, 03 = CWR, (0C = PKT - Used with LDG Z817 autotuner)
 There are a number of other values, but they are not applicable to the Juma TRX-2

 Read RX Status
 Command To Transceiver	: 00 00 00 00 E7
 Transceiver Response	: xx 00 00 00 00

 xx is the data byte, and the bits have the following meanings:

	7	Squelch Status 0 = OFF, Signal present, 1 = ON, Receiver squelched
	6	CTCSS/DCS code. For Juma = 0
	5	Discriminator Centering, for SSB/CW = 0
	4	Dummy Data 0
	3	S-Meter Bit 3
	2	S-Meter Bit 2
	1	S-Meter Bit 1
	0	S-Meter Bit 0

 The S-meter value is coded into 15 levels from S0 to S9+40dB

 Read RX Status
 Command To Transceiver	: 00 00 00 00 F7
 Transceiver Response	: xx 00 00 00 00

 xx is the data byte, and the bits have the following meanings:

	7	PTT Status 0 = TX, 1 = RX
	6	High SWR 0 = OFF, 1 = ON
	5	Split Mode 0 = ON, 1 = OFF
	4	Dummy Data 0
	3	PWR Meter Bit 3
	2	PWR Meter Bit 2
	1	PWR Meter Bit 1
	0	PWR Meter Bit 0

 The PWR-meter value is coded into 15 levels from 0 - 100W?

 Split Mode ON
 Command To Transceiver	: 00 00 00 00 02

 Split Mode OFF
 Command To Transceiver	: 00 00 00 00 82

 VFO Select (Toggle VFO-A, VFO-B)
 Command To Transceiver	: 00 00 00 00 81

 Set Operating Mode
 Command To Transceiver	: xx 00 00 00 07
 Where xx has the following meanings: 00 = LSB, 01 = USB, 02 = CW, 03 = CWR

 Set Operating Frequency
 Command To Transceiver	: aa bb cc dd 01
 Assume the frequency is: 14.23456MHz, aa = 01, bb = 42, cc = 34, dd = 56

 PTT ON (Transmit)
 Command To Transceiver	: 00 00 00 00 08

 PTT OFF (Receive)
 Command To Transceiver	: 00 00 00 00 88
 
 Lock VFO
 Command To Transceiver	: 00 00 00 00 00

 Unlock VFO
 Command To Transceiver	: 00 00 00 00 80

 Set Tuning Rate
 Command To Transceiver	: xx 00 00 00 E0
 Where xx has the following meanings: 00 = Slow - 1Hz, 01 = Medium - 10Hz, 02 = Fast - 100Hz, 03 = Very Fast - 1kHz
 
 Set Receiver Bandwidth
 Command To Transceiver	: xx 00 00 00 E8
 Where xx has the following meanings: 00 = Narrow, 01 = Wide

 Extended Display
 Command To Transceiver	: xx 00 00 00 F8
 Where xx has the following meanings: 00 = Normal, 01 = Power, 02 = SWR, 03 = Voltmeter, 04 = Ammeter
*/

#include "juma-trx2.h"				// Board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "trx2_eeprom.h"			// Get EEPROM structure definitions

// External Functions
extern void putch(char);			// UART I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern void set_bit(int);
extern void clr_bit(int);
extern void clear_buffer();			// Defined in serial_trx2.c

// External Data
extern unsigned char cmd_buf[];		// Defined in serial_trx2.c
extern int cmd_buf_idx;				// Defined in serial_trx2.c
extern long freq_in_use;			// Frequency (DDS set value) currently in use
extern int vfo_idx;					// VFO index, A = 0, B = 1, if (Split) then RX = 0, TX = 1
extern int split_flag;				// Split Mode 0 = Simplex, 1 = Split
extern int sub_page0;				// Extended Display index
extern int s_meter;					// Scaled S-meter value 0 ... 48
extern int pwr_meter;				// Scaled S-meter value 0 ... 48
extern int alarm_flag;				// SWR alarm flag
extern int PTT_IN;					// PTT status
extern int squelch_on;				// Squelch on flag, 0 = Off, 1 = On

// Timeout moved to timer system
extern int cmd_timeout;				// RS-232 command time-out, 0 = Time-out.

#define CMD_TIMEOUT		200			// Yaesu spec = 200ms, 100
#define CMD				4			// Opcode table index
#define MAX_YAESU_CMD	CMD + 1
#define YAESU_BUFF_SIZE	6

// EEPROM structures
// User Configuration Data
extern union
	{
	struct defval defval;
	} eeprom;

// Module Variables
static int old_vfo;					// Temporary storage for split command

// Global Variables
unsigned char last_set_freq[YAESU_BUFF_SIZE];	// Last frequency data received from CAT 

int bcd2bin(int num)
    {
    int n;

    n = num >> 4;
    n *= 10;
    return n + (num & 0x0F);
    }

int bin2bcd(int num)
    {
    int v;

    v = num / 10;
    num -= v * 10;
    v <<= 4;
    return (v + num);
    }

// Convert CAT frequency data to JUMA-TRX2 format
unsigned long convert_cat_freq(void)
	{
	unsigned long freq = 0UL;
	int i = 0;

	last_set_freq[CMD] = 0;		// Clear convert flag

	do	{
		freq *= 100UL;
		freq += (unsigned long)bcd2bin(last_set_freq[i]);
		} while(++i < CMD);

	freq *= 10UL;

	if(freq > 30000000UL)		// Check limit
		freq = 30000000UL;

	return freq;				// Store frequency in selected VFO, frequency is in decimal
	}

/*
 Yaesu FT-897 CAT Interface Emulator
 Subset of Yaesu commands

 The emulator uses a command timer of 200mS, contained in the global variable: cmd_timeout
 declared in module: timers_pwm.c This variable is decremented in function: _T3Interrupt()
 The timer is used for the frequency conversion, as some CAT programs do not take account
 of the Yaesu specification, and this timer is used to avoid permanently tying up the serial
 port interrupt handler.
  
 When the emulation cycle is started, the timer is loaded with the 200mS time-out value, and
 this is decremented in the 1mS timer interrupt. If a set frequency command is received,
 the data bytes from the yaesu_command[] buffer are copied to the holding array, last_set_freq[].
 When the timer has expired the yaesu_command[] buffer is cleared, and the cmd_timeout is set
 to -1 to act as a flag. This is checked immediately after, and now the frequency copied to the
 holding buffer is converted and sent to the transceiver.

 The start of a cycle checks the kbhit() for the presence of characters in the UART buffer.
 If a character is present, this indicates that a command is being sent. The command timer
 is reset, and the character is inserted into the yaesu_command[] buffer, the buffer index
 is incremented, and a check performed to see if we have received all the bytes (5).
 If we have received less than 5 bytes, the function simply returns and waits for another
 character. When the next character is received, the process repeats, we reset the timer,
 and insert the character into the next buffer location. Eventually we will have received all
 5 bytes. At this point we reset the buffer index ready for the next command, and now examine
 the command byte in yaesu_command[4].

 Most commands either have no parameters, for example, turn RIT On or Off, or have only 1
 parameter, for example, set operating mode. Other commands simply read data from the
 transceiver, for example, read TX status.

 For those commands that have no parameters, the action is performed, and the function simply
 returns after sending the Yaesu ACK byte (NUL).

 For those commands that read status or data, if it is a single byte, then the data is
 collected, the data byte encoded, and simply transmitted, and then the Yaesu ACK byte sent.

 If the status or data is a multi-byte value, for example, receiver frequency and mode, the
 data is collected and encoded, and placed in the yaesu_command[] buffer. Once all the data
 has been encoded, this is then transmitted one byte at a time. In this case, no ACK is transmitted.

 There are some exceptions, the E7 - Read Receiver S-Meter & Status, returns FF if the transceiver
 is in the TX state, and the F7 - Read Transmitter Power & Status also returns FF if the
 transceiver is in the RX state.
*/
void serial_yaesu(void)
	{
	int i;
	unsigned long freq, v, t;
	unsigned char ctemp;

// CAT Serial Time-out
	if(!cmd_timeout)					// cmd_timeout handled in the timer system
		{
		clear_buffer();					// Clear cmd_buf[] buffer, cmd_buf_idx left at zero on exit
		cmd_timeout = -1;				// Time-out noticed
		}

// Check if CAT frequency conversion needed
	if((last_set_freq[CMD]) && cmd_timeout == -1)			// Convert & set CAT frequency
		eeprom.defval.vfo[vfo_idx] = convert_cat_freq();	// Do it only after serial time-out, avoid congestion

// Handle input if any
	if(kbhit())
		{
		cmd_timeout = CMD_TIMEOUT;		// Reload time-out timer
		cmd_buf[cmd_buf_idx++] = getch();	// Get character, add to buffer and increment index

		if(cmd_buf_idx < MAX_YAESU_CMD)	// MAX_YAESU_CMD = 5
			return;						// Command not ready yet

		cmd_buf_idx = 0;				// Set RX pointer for new receive

// Encode & process commands
		switch(cmd_buf[CMD])			// Check and execute the command
			{
// Undefined Commands
			default:
				clear_buffer();			// Clear CAT structure
				putch(0xF0);			// Send Yaesu NAK, 29.01.2009
			break;

// Read commands
// Read RX Frequency & Mode
			case 0x03:
				freq = freq_in_use;		// Frequency is already in decimal format
				v = 10000000UL;
// Convert frequency to BCD
				i = 0;

				do	{
					t = freq / v;
					cmd_buf[i] = bin2bcd((int)(t));
					freq -= (t * v);
					v /= 100UL;
					} while(++i < CMD);

// Construct mode bits, TRX2 modes:  0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 0x0C = Tune (Yaesu PKT mode)
				if(eeprom.defval.mode[vfo_idx] < TUNE)
					cmd_buf[CMD] = eeprom.defval.mode[vfo_idx]; // TRX2 mode is the same
				else
					cmd_buf[CMD] = 0x0C;// Treat Tune as PKT mode
// Transmit packet
// This section was originally a separate function, send_cat(), but was moved here as it was only called once.
// -----------------------------------------------------------------------------------------
				i = 0;

				do	{								// Send Yaesu CAT formatted frequency packet
					putch(cmd_buf[i]);				// Send five bytes, [0]/P1 first
					} while(++i < MAX_YAESU_CMD);
// -----------------------------------------------------------------------------------------
			break;

// Read RX Status
			case 0xE7:
				if(PTT_IN)					// If RX
					{
					ctemp = s_meter / 3;	// Scale s-meter down to 0 ... 15

					if(ctemp > 15)
						ctemp = 15;			// Saturate

					if(squelch_on)
						ctemp |= 0x80; 		// Squelch on, no signal
					}
				else
//					ctemp = 0;				// No S-meter value & RX flags if TX on
					ctemp = 0xFF;			// Yaesu 897 CAT protocol oddity, 29.01.2009

				putch(ctemp);				// Send only one byte
			break;

// Read TX Status
			case 0xF7:
				if(!PTT_IN)					// If TX on
					{
					ctemp = pwr_meter / 3;	// Scale power meter down to 0 ... 15

					if(ctemp > 15)
						ctemp = 15;			// Saturate

					ctemp |= 0x80;			// Set PTT bit if on

					if((alarm_flag & 0x02))	// Set HI SWR flag
						ctemp |= 0x40;

					if(split_flag)			// Split Mode On?
					 	ctemp |= 0x20;		// Set split on flag
					}
				else
//					ctemp = 0;				// No PWR meter value, no TX related flags
					ctemp = 0xFF;			// Yaesu 897 CAT protocol oddity, 29.01.2009

				putch(ctemp);				// Send only one byte response
			break;

// Set commands
// VFO Lock ON
			case 0x00:
				set_bit(vfo_idx);			// Lock the current VFO
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

// VFO Lock OFF
			case 0x80:
				clr_bit(vfo_idx);			// Unlock the current VFO
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

// Set Frequency
			case 0x01:
/*
 Some programs do not respect Yaesu protocol 200ms serial time-out data
 for fast receive just copy data, convert later
 last_set_freq[CMD] used as a flag (Yaesu command: 0x01) for later frequency conversion
*/
				i = 0;

				do	{								// Just copy freq data
					last_set_freq[i] = cmd_buf[i];
					} while (++i < MAX_YAESU_CMD);

				putch(0x00);						// Send Yaesu ACK, 29.01.2009
			break;

// Split ON
			case 0x02:
				old_vfo = Select_VFO; 				// Save current status
				Select_VFO = SPLIT_MODE;			// Set split mode
//				putch(0x02);						// Echo command, send only one byte
				putch(0x00);						// Send Yaesu ACK, 29.01.2009
			break;

// Split Off
			case 0x82:
				if(old_vfo < 2)
					Select_VFO = old_vfo;			// Restore selection before split
				else
					Select_VFO = VFO_A;				// If old state was Split select VFO-A
//				putch(0x82);						// Echo command, send only one byte
				putch(0x00);						// Send Yaesu ACK, 29.01.2009
			break;

// RIT On
			case 0x05:
				RIT_On_Off = ON;
				putch(0x00);						// Send Yaesu ACK, 29.01.2009
			break;

// RIT Off
			case 0x85:
				RIT_On_Off = OFF;
				putch(0x00);						// Send Yaesu ACK, 29.01.2009
			break;

// Set Operating mode, P1 = mode bits
/*
 The special case when the transceiver is being used with the LDG Z-817 autotuner assumes the following:

 1.	When the TUNE button of the autotuner is pressed, the autotuner requests the current RX mode and frequency
	with the 00 00 00 00 03 command.
 2.	The Juma replies with the current mode and frequency which the autotuner stores.
 3.	The autotuner then requests the PKT mode with the command: 0C 00 00 00 07
 4.	The Juma responds by selecting the TUNE mode.
 5.	The autotuner initiates a transmit operation by sending the PTT ON command: 00 00 00 00 08
 6.	The Juma responds by switching to transmit.
 7.	At the completion of the tuning cycle the autotuner sends the PTT OFF command: 00 00 00 00 88
 8.	The Juma responds by switching back to receive.
 9.	The autotuner restores the previous mode with the set mode command: xx 00 00 00 07
	where xx can have the value 00 = LSB, 01 = USB, 02 = CW, 03 = CWR.

 This code did not prove to be successful. It provoked an intermittent Address Error, as well as returning the
 transceiver to the wrong mode. Without an autotuner and a serial port analyser, I don't see how I can proceed
 to try and correct these problems. A.Ryan 17/SEP/12
*/
			case 0x07:
				i = cmd_buf[0];				// This saves repeated address calculation

				if(i == 0x0C) i = TUNE;		// Treat PKT as TUNE

				if(i <= TUNE)
					{
					eeprom.defval.mode[vfo_idx] = i;
					ctemp = 0x00;
					}
				else ctemp = 0xF0;			// Unknown/Illegal mode

				putch(ctemp);
			break;

// PTT ON
			case 0x08:
				PTT_OUT = 1;				// TX on
//				putch(0x08);				// Echo only one byte
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

// PTT OFF
			case 0x88:
				PTT_OUT = 0;				// TX off
//				putch(0x88);				// Echo only one byte
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

// Toggle VFO select 0 = A, 1 = B
			case 0x81:
/*
 Do not be tempted to substitute eeprom.defval.vfo_sel ^= 1;
 If split mode is in operation, then eeprom.defval.vfo_sel = 2, in which case instead of
 dropping the split mode and selecting VFO-A, you will select VFO-D A.Ryan 20/NOV/2011
*/
				if(Select_VFO != VFO_A)		// VFO settings, 0 = A, 1 = B
					Select_VFO = VFO_A;
				else
					Select_VFO = VFO_B;

				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

			case 0xE0:						// Set Tuning Rate - Not part of standard Yaesu protocol!
				Tuning_Step = cmd_buf[0] % 4;
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

			case 0xF8:						// Set Extended Display - Not part of standard Yaesu protocol!
				sub_page0 = cmd_buf[0] % 5;
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;

			case 0xE8:						// Set Receiver Bandwidth - Not part of standard Yaesu protocol!
				RX_Bandwidth = cmd_buf[0] & 0x01;
				putch(0x00);				// Send Yaesu ACK, 29.01.2009
			break;
/*
 Unknown commands, response from the hat to get the HRD work
 No idea what commands 00 xx 00 00 BB do. With PC port monitor found that they send a two byte response
 We send just two nulls to keep HRD running
*/
			case 0xBB:
				putch(0x00);				// Echo two bytes, 897 seems to do so
				putch(0x00);
			break;
			}	// Case
		}	// If
	}

