// JUMA-TRX2 protocol
// Juha Niinikoski, OH2NLT 06.07.2006
// TX / PTT control added to JUMA-TRX2 protocol 17.04.2009 v1.06 test

/*
 Juma-TRX2 & PA100D Command Protocol
 Commands take the form of either a query or a command.

 QUERY
 The query format is: =?c\n\r, or ?c\n\r
 Where 'c' has the following meanings:
 A	Filters
 TRX2 Response: =An\n\r
 n = 0 - 1 0 = Narrow, 1 = Wide

 B	Band Data
 TRX2 Response: =Bn\n\r
 n = 1 through 9 to signify the bands from 160m - 10m, 1 = 160m, 9 = 10m

 C	Speech Processor
 TRX2 Response: =Cn\n\r
 n = 0 - 1 0 = Off, 1 = On

 F	Frequency
 Extended Command:	=?FA\n\r | =?FB\n\r
 TRX2 Response: =Fnnnnnnnn\n\r or FAnnnnnnnn\n\r or FBnnnnnnnn\n\r
 FA = VFO-A, FB = VFO-B
 n = 0 - 9, Ex: 12.345678MHz = =F12345678\n\r

 G	AGC Speed
 TRX2 Response: =Gn\n\r
 n = 0 - 1 0 = Slow, 1 = Fast

 I	Auto-Inform
 TRX2 Response: =In\n\r
 n = 0 - 3 0 = Auto-Inform Disable, 1,2,3 = Auto-Inform On, 1 = 50mSec, 2 = 100mSec, 3 = 150mSec Latency times

 J	CW Shift
 TRX2 Response: =Jnnnn\n\r
 n = 0 - 9 Ex: 700Hz Shift =J700\n\r

 L	Current VFO Lock Status
 Extended Command:	=?LA\n\r | =?LB\n\r
 TRX2 Response: =Ln\n\r or LAn\n\r | =LBn\n\r
 LA = VFO-A, LB = VFO-B
 n = 0 - 1, 0 = Unlocked 1 = Locked

 M	Mode
 Extended Command:	=?MA\n\r | =?MB\n\r
 TRX2 Response: =Mn\n\r | MAn\n\r | MBn\n\r
 MA = VFO-A, MB = VFO-B
 n = 0 - 4 for LSB, USB, CW, CWR, or Tune

 Q	Tuning Rate
 TRX2 Response: =Qn\n\r
 n = 0 - 2 0 = 1Hz, 1 = 10Hz, 2 = 100Hz

 R	RIT Status
 TRX2 Response: =Rn\n\r
 n = 0/1 0 = Off, 1 = On

 S	S-Meter
 TRX2 Response: =Snn\n\r
 nn = 0 - 48 In Receive, this is the S-meter, in Transmit, it is the power meter.

 T	PTT Status
 TRX2 Response: =Tn\n\r
 n = 0/1 0 = RX, 1 = TX

 U	Firmware Version
 TRX2 Response:	=U1.07w\n\r

 V	VFO Number
 TRX2 Response: =Vn\n\r
 n = 0 - 1 0 = VFO-A, 1 = VFO-B

 W	Filter Bandwidth
 Extended Command: =?WN\n\r | =?WW\n\r
 WN = Narrow, WW = Wide
 TRX2 Response: =Wnnnn\n\r | WWnnnn\n\r
 Note: If the filter queried is the current filter, then even using the extended command, the response will be =Wnnnn\n\r.
 nnnn = filter B/W, Hz

 Y	Extended Display
 TRX2 Response: =Yn\n\r
 n = 0 - 4, 0 = Normal, 1 = Power, 2 = SWR, 3 = Voltmeter, 4 = Ammeter

 COMMANDS
 The command format is: =cn\n\r
 Where 'c' and 'n' have the following meanings:

 A	Select Filter
 Command:	=An\n\r
 n = 0 - 1, 0 = Narrow, 1 = Wide

 B	Band Change
 Command:	=Bn\n\r
 n = 1 - 9, 1 = 160m, 2 = 80m, 3 = 40m, 4 = 30m, 5 = 20m, 6 = 17m, 7 = 15m, 8 = 12m, 9 = 10m

 C	Speech Processor
 Command:	=Cn\n\r
 n = 0 - 1 0 = Off, 1 = On

 E	Equalise VFO Settings
 Command:	En\n\r
 n = 0 - 1 0 - VFO-A = VFO-B, 1 - VFO-B = VFO-A

 F	Set Frequency
 Command:	=Fnnnnnnnn\n\r
 Extended Command:	=FAnnnnnnnn\n\r | =FBnnnnnnnn\n\r
 A = VFO-A Frequency set, B = VFO-B Frequency set
 n = 00000000 - 30000000 Frequency must be specified to 1Hz

 G	AGC Speed
 Command:	=Gn\n\r
 n = 0 - 1, 0 = Slow, 1 = Fast

 I	Auto-Inform
 Command:	=In\n\r
 0 = Auto-Inform Off, 1/2/3 Auto-Inform On, Latency Times: 1 = 50mSec, 2 = 100mSec, 3 = 150mSec

 L	Lock Current VFO
 Command:	=Ln\n\r
 Extended Command:	=LAn\n\r | =LB\n\r
 A = VFO-A Lock/Unlock, B = VFO-B Lock/Unlock
 n = 0 - 1, 0 = Unlocked, 1 = Locked

 M	Set Mode
 Command:	=Mn\n\r
 Extended Command:	=MAn\n\r | =MB\n\r
 MA = VFO-A Mode, MB = VFO-B Mode
 n = 0 - 3 0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 4 = Tune

 P	Remote Power OFF
 Command:	=Pn\n\r
 n = 0 - 1 0 = Power OFF, no save. 1 = Power OFF, save User Configuration settings.

 Q	Tuning Rate
 Command:	=Qn\n\r
 n = 0 - 2, 0 = 1Hz, 1 = 10Hz, 2 = 100Hz

 R	Set RIT
 Command:	=Rn\n\r
 n = 0 - 1, 0 = Off, 1 = On

 T	Set TX/RX
 Command:	=Tn\n\r
 n = 0 - 1, 0 = RX, 1 = TX

 V	Select VFO
 Command:	=Vn\n\r
 n = 0 - 2, 0 = VFO-A, Simplex, 1 = VFO-B, Simplex, 2 = VFO-A Rx, VFO-B Tx, Split Mode

 W	Set Filter B/W
 Command:	=Wnnnn\n\r
 Extended Command:	=WNnnnn | =WWnnnn\n\r
 Where N = Narrow, W = Wide
 n = 500 - 2500

 X	Exchange VFO Settings
 Command:	=X\n\r

 Y	Extended Display
 Command:	=Yn\n\r
 n = 0 - 4, 0 = Normal, 1 = Power, 2 = SWR, 3 = Voltmeter, 4 = Ammeter
*/

#include <stdio.h>					// Added - 5B4AIY
#include <stdlib.h>					// Added - 5B4AIY
#include <ctype.h>					// Added - 5B4AIY
#include "juma-trx2.h"				// Board definitions
#include "trx2_eeprom.h"			// Get EEPROM structure definitions

// Module Variables
#define CMD_BUF_SIZE	16
#define MAX_TRX2_CMD	CMD_BUF_SIZE - 1

unsigned char cmd_buf[CMD_BUF_SIZE];// Command buffer - Also used by Kenwood and Yaesu CAT modules.
int cmd_buf_idx = 0;				// Buffer fill pointer - Also used by Kenwood and Yaesu CAT modules.
static int band_update;				// Flag used to update PA-100D band data

// These variables are used by the Auto-Inform function to save the current status
long vfo_freq[2];					// Current VFO-A frequency
int mode[2];						// Transceiver Mode VFO-A. 0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 4 = Tune
int filter_bw[2];					// Filter bandwidths
int filter;							// Receive filter 0 = Narrow, 1 = Mid, 2 = Wide
int vfo;							// VFO in use 0 = VFO-A, 1 = VFO-B, 2 = Split Mode
int tuning_rate;					// Current Tuning Rate 0 = 1Hz, 2 = 10 Hz, 3 = 100Hz
int rit_on_off;						// RIT On/Off 0 = Off, 1 = On
int rit_freq;						// RIT Offset +/- 1000Hz maximum
long cw_shift;						// Current CW offset frequency 300Hz - 1100Hz
int s_mtr;							// Current S-meter
int power_mtr;						// Current power meter
int vfo_lock_status[2];				// Current VFO-A lock status 0 = unlocked, 1 = locked
int ptt_in;							// Current Tx/Rx status. 0 = Tx, 1 = Rx
int speech_comp;					// Speech Processor On/Off 0 = Off, 1 = On
int agc_spd;						// AGC Fast/Slow 0 = Slow, 1 = Fast
int auto_inform;					// Auto-Inform flag 0 = Off, 1 = On
int latency_count;					// Set when Auto-Inform is enabled to 50/100/150 mSec.
int A_equals_B_flag;				// Set in main module when setting A=B, or copying data from A to B, or B to A.

const char filter_char[] = {'N', 'W'};
const char VFO_status_msg[] = {"=V%c:F%liM%iL%i\n\r"};
const char VFO_status[] = {"=F%li\n\r=M%i\n\r=L%i\n\r"};
const char Extended_BW_msg[] = {"=W%c%li\n\r"};
const char BW_msg[] = {"=W%li\n\r"};
const char Filter_BW_msg[] = {"=A%i:W%li\n\r"};
const char VFO_msg[] = {"=V%i\n\r"};
const char VFO_freq_msg[] = {"=F%li\n\r"};
const char Lock_msg[] = {"=L%i\n\r"};
const char Extended_Lock_msg[] = {"=L%c%i\n\r"};
const char Mode_msg[] = {"=M%i\n\r"};
const char Extended_Mode_msg[] = {"=M%c%i\n\r"};
const char Tuning_Rate_msg[] = {"=Q%i\n\r"};
const char Meter_msg[] = {"=S%i\n\r"};
const char Tx_msg[] = {"=T1\n\r"};
const char Rx_msg[] = {"=T0\n\r"};
const char Print_RIT[] = {"=R%i:%+i\n\r"};
const char CW_Offset_Cmd[] = {"=J%i\n\r"};
const char Speech_Proc_Cmd[] = {"=C%i\n\r"};
const char AGC_Cmd[] = {"=G%i\n\r"};
const char S_Meter_Cmd[] = {"=T0:S%i\n\r"};
const char TX_Pwr_Cmd[] = {"=T1:S%i\n\r"};
const char Band_Cmd[] = {"=B%i\n\r"};
const char Speech_Proc_Mode[] = {"=C%i\n\r"};
const char Ext_Freq_Cmd[] = {"=F%c%li\n\r"};
const char Auto_Inf_Status[] = {"=I%i\n\r"};
const char Firmware_Ver[] = {"=U: %s\n\r"};
const char Set_Freq_Cmd[] = {"=F%li\n\r"};
const char Extended_Display[] = {"=Y%1i\n\r"};
const char Remote_Pwr_Off[] = {"Remote Power OFF"};
const char Settings_Saved[] = {" Settings Saved "};

// External References
// External Functions
extern int kbhit(void);
extern unsigned char getch(void);
extern void ms_delay(int);
extern void lcd_cmd(unsigned char);
extern void lcd_putst(register const char *);
extern void save_defval(void);
extern void set_bit(int);
extern void clr_bit(int);
extern int check_bit(int);
extern void max_min(int *, int, int);
extern void Switch_Off(void);

extern int PTT_IN;							// Atomic copy of PTT_IN_IO input / TX on, 0 = Active
extern int s_meter;							// Scaled S-meter value 0 ... 48
extern int pwr_meter;						// Scaled PWR-meter value 0 ... 48
extern int vfo_idx;							// VFO index, A = 0, B = 1, if(Split) then  RX = 0, TX = 1
extern const unsigned long amateur_bands[];	// 0 = 160m, 8 = 10m
extern const char ver[];
extern int vfo_lock;						// VFO lock status, 1 = Locked
extern int rit;								// RIT offset frequency
extern int sample_flag;						// Decremented in timers_pwm.c
extern int sub_page0;						// Display page index

// SCAF filter
extern const int scaf_min[];				// Filter tuning limits
extern const int scaf_max[];

// User Configuration Data
extern union
	{
	struct defval defval;
	} eeprom;

extern int rf_filt_idx;						// TRX2 band data
extern long freq_in_use;					// Frequency (DDS set value) currently in use
extern unsigned int pa100_tx_tmr;			// PA100 max band change rate

#define MAX_TX_RATE	100						// Max TX rate (ms) to send data to PA-100D

// Clear buffer
void clear_buffer(void)
	{
	cmd_buf_idx = CMD_BUF_SIZE;

	do	{
		cmd_buf[--cmd_buf_idx] = 0;
		} while(cmd_buf_idx);		// This also resets cmd_buf_idx
	}

// Some I/O functions
// Convert single digit number from command buffer
int get_num(int index, int limit)
	{
	return (cmd_buf[index] - '0') % limit;
	}

// Auto-Inform logic
void reset_state(void)
	{
	mode[0] = 0xFF;					// Transceiver Mode VFO-A. 0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 4 = Tune
	mode[1] = 0xFF;					// Transceiver Mode VFO-B. 0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 4 = Tune
	filter_bw[0] = 0xFFFF;			// Narrow
	filter_bw[1] = 0xFFFF;			// Wide
	filter = 0xFF;					// Receive filter 0 = Narrow, 1 = Wide
	vfo = 0xFF;						// VFO in use 0 = VFO-A, 1 = VFO-B, 2 = Split Mode
	tuning_rate = 0xFF;				// Current Tuning Rate 0 = 1Hz, 2 = 10 Hz, 3 = 100Hz
	rit_on_off = 0xFF;				// RIT On/Off 0 = Off, 1 = On
	rit_freq = 2000;				// RIT Offset +/- 1000Hz maximum, set to this value to force an update.
	cw_shift = 2000;				// Current CW offset frequency 300Hz - 1100Hz
	vfo_freq[0] = 40000000;			// Current VFO-A frequency
	vfo_freq[1] = 40000000;			// Current VFO-B frequency
	s_mtr = 100;					// Current S-meter
	power_mtr = 100;				// Current power meter
	vfo_lock_status[0] = 0xFF;		// Current VFO-A lock status 0 = unlocked, 1 = locked
	vfo_lock_status[1] = 0xFF;		// Current VFO-B lock status 0 = unlocked, 1 = locked
	ptt_in = 0xFF;					// Current Tx/Rx status. 0 = Tx, 1 = Rx
	speech_comp = 0xFF;				// Speech Processor On/Off 0 = Off, 1 = On
	agc_spd = 0xFF;					// AGC Fast/Slow 0 = Slow, 1 = Fast
	sample_flag = 0;				// Reset sample_flag to ensure no samples occur
	latency_count = 0;				// Reset latency count
	A_equals_B_flag = FALSE;
	}

void chk_vfo(int index)
	{
	int i;

	if(vfo_freq[index] != eeprom.defval.vfo[index])
		{
		vfo_freq[index] = eeprom.defval.vfo[index];
		printf(VFO_freq_msg, eeprom.defval.vfo[index]);
		}

	if(mode[index] != eeprom.defval.mode[index])
		{
		mode[index] = eeprom.defval.mode[index];
		printf(Mode_msg, eeprom.defval.mode[index]);
		}

	i = check_bit(index);

	if(vfo_lock_status[index] != i)
		{
		vfo_lock_status[index] = i;
		printf(Lock_msg, vfo_lock_status[index]);
		}
	}

void chk_state(int first_use_flag)
	{
	int i, j, local_vfo;

	if(sample_flag) return;								// If timer has not timed-out, then return
// This is necessary because the main loop does not set vfo_idx until the TRX-2 protocol handler returns, and
// thus the VFO in use does not get correctly set. Re-writing the main loop to update before calling this function
// is inadvisable.
	local_vfo = Select_VFO % 2;
	sample_flag = latency_count;						// Set timer to desired latency time (50/100/150mSec)
	i = 1;

	if(first_use_flag)
		{
		if(local_vfo == 1) i = 0;

		j = check_bit(i);
		printf(VFO_status_msg, i + 'A', eeprom.defval.vfo[i], eeprom.defval.mode[i], j);
		printf(VFO_status, eeprom.defval.vfo[i], eeprom.defval.mode[i], j);
		}

	if(vfo != Select_VFO)									// Check current VFO selection
		{
		j = check_bit(local_vfo);
		vfo = Select_VFO;
		printf(VFO_msg, Select_VFO);
		printf(VFO_status_msg, local_vfo + 'A', eeprom.defval.vfo[local_vfo], eeprom.defval.mode[local_vfo], j);
		vfo_freq[local_vfo] = eeprom.defval.vfo[local_vfo];
		mode[local_vfo] = eeprom.defval.mode[local_vfo];
		vfo_lock_status[local_vfo] = j;
		printf(VFO_status, eeprom.defval.vfo[local_vfo], eeprom.defval.mode[local_vfo], j);
		}

	chk_vfo(local_vfo);

	if(A_equals_B_flag)			// This flag can have the value 1 if copying B to A, or 2 if copying A to B
		{
		i = A_equals_B_flag - 1;
		A_equals_B_flag = 0;	// Reset flag
		printf(VFO_status_msg, i + 'A', eeprom.defval.vfo[i], eeprom.defval.mode[i], check_bit(i));	// Changed VFO, new frequency, mode, and lock status
		}

	i = 0;

	do	{
		if(filter_bw[i] != eeprom.defval.scaf_filters[i])
			{
			filter_bw[i] = eeprom.defval.scaf_filters[i];

			if(i == RX_Bandwidth) printf(BW_msg, (75000L / filter_bw[i]));
			else printf(Extended_BW_msg, filter_char[i], (75000L / filter_bw[i]));
			}
		} while (++i < 2);

	if(filter != RX_Bandwidth)	// Check current receive filter selection
		{
		filter = RX_Bandwidth;
		printf(Filter_BW_msg, filter, (75000L / eeprom.defval.scaf_filters[filter]));	// Send audio filter #
		}

	if(tuning_rate != Tuning_Step)						// Check current VFO tuning rate
		{
		tuning_rate = Tuning_Step;
		printf(Tuning_Rate_msg, Tuning_Step);
		}

	if((rit_on_off != RIT_On_Off) || (rit_freq != rit))	// Check current RIT On/Off Status and frequency
		{
		rit_on_off = RIT_On_Off;
		rit_freq = rit;
		printf(Print_RIT, rit_on_off, rit);
		}

	if(cw_shift != eeprom.defval.cw_shift)				// Check current CW offset
		{
		cw_shift = eeprom.defval.cw_shift;
		printf(CW_Offset_Cmd, eeprom.defval.cw_shift);
		}

	if(ptt_in != PTT_IN)								// Check transmit/receive status
		{
		ptt_in = PTT_IN;

		if(ptt_in) printf(Rx_msg);
		else printf(Tx_msg);
		}

	if(speech_comp != Speech_Processor)		// Check current speech processor setting
		{
		speech_comp = Speech_Processor;
		printf(Speech_Proc_Cmd, Speech_Processor);
		}

	if(agc_spd != eeprom.defval.agc_speed)				// Check AGC speed setting
		{
		agc_spd = eeprom.defval.agc_speed;
		printf(AGC_Cmd, eeprom.defval.agc_speed);
		}

	if(s_mtr != s_meter)								// Check current S-meter reading
		{
		s_mtr = s_meter;
		printf(S_Meter_Cmd, s_mtr);
		}

	if(power_mtr != pwr_meter)							// Check current transmit power meter reading
		{
		power_mtr = pwr_meter;
		printf(TX_Pwr_Cmd, pwr_meter);
		}
	}

// TRX2 protocol parser
// Send band data
void send_band_data(void)
	{
	int i;
// rf_filt_idx has values from 0 - 11
	i = rf_filt_idx - 2;			// Adjust TRX2 filter index to match PA100 band select

	if(i < 0) i = 0;

	printf(Band_Cmd, i);
	}

// Set selected filter width
void set_filter_width(int index, int filter)
	{
	int w;

	w = atoi(cmd_buf + index);
	max_min(&w, 2500, 500);
	eeprom.defval.scaf_filters[filter] = 75000L / w;// Convert to timer units
	}

// Parse TRX2 message
void serial_trx2(int freq_change_flag)
	{
	static int old_rf_filt_idx;						// Initialised to zero by default.
	unsigned long f = 0;
	int b, i;
	char c, a;

	if(auto_inform) chk_state(0);

// Detect TRX2 band changes & send band data to the PA100
	if(rf_filt_idx != old_rf_filt_idx)				// Band changed
		{
		if(!pa100_tx_tmr)							// If timer has timed out
			{
			pa100_tx_tmr = MAX_TX_RATE;				// Load inhibit timer (100mS)
			old_rf_filt_idx = rf_filt_idx;			// Save current situation
			send_band_data();						// Send band data
			}
		}

	if(PTT_IN == RECEIVE && band_update == TRUE)	// In Receive, so
		band_update = FALSE;						// PA-100D band data uncertain

	if(PTT_IN == TRANSMIT && band_update == FALSE)	// In Transmit and uncertain band data
		{
		send_band_data();
		band_update = TRUE;							// Band data now certain
		}

// Process input
	if(kbhit())
		{
		c = getch();								// Save character

// Ignore characters that not CR, =, ?, or other non-alphanumeric.
		if(!((isalnum(c)) || (c == 0x0D) || (c == '=') || (c == '?'))) return;

// Capture start sync
		if(c == '=' || c == '?') cmd_buf_idx = 0;	// Sync with action characters

// Check if buffer ready, interpret message
		if(c == 0x0D)								// CR = ready
			{
			a = cmd_buf[2];							// Letter if extended command, or a number if a parameter

			if(cmd_buf[0] == '?')					// Query Status
				{
				switch(cmd_buf[1])
					{
					case 'A':						// Receiver Filter & Bandwidth
						printf(Filter_BW_msg, RX_Bandwidth, (long)(75000L / eeprom.defval.scaf_filters[RX_Bandwidth]));	// Send audio filter bandwidth
					break;

					case 'B':						// Current Band
						send_band_data();			// Send band data answer, B0 = < 1MHz, B1 = 160m to B9 = 10m
					break;

					case 'C':						// Speech Processor (Compressor) setting, 0 = Off, 1 = On
						printf(Speech_Proc_Cmd, Speech_Processor);	// Send mode data
					break;

					case 'F':						// VFO Frequency
						if(a == 'A' || a == 'B')					// Extended Command
							printf(Ext_Freq_Cmd, a, eeprom.defval.vfo[(int)(a - 'A')]);
						else
							printf(VFO_freq_msg, freq_in_use);		// Standard Response
					break;

					case 'G':						// AGC Speed 0 = Slow, 1 = Fast
						printf(AGC_Cmd, eeprom.defval.agc_speed);	// Send mode data
					break;

					case 'I':						// Auto-Inform Status
						printf(Auto_Inf_Status, auto_inform);
					break;

					case 'J':						// CW Offset
						printf(CW_Offset_Cmd, eeprom.defval.cw_shift);
					break;

					case 'L':						// VFO Lock Status
						if(a == 'A' || a == 'B')					// Extended Command
							printf(Extended_Lock_msg, a, check_bit((int)a - 'A'));// Extended Response
						else
							printf(Lock_msg, check_bit(vfo_idx));	// Standard Response
					break;

					case 'M':						// VFO Mode
						if(a == 'A' || a == 'B')					// Extended Command
							printf(Extended_Mode_msg, a, eeprom.defval.mode[(int)a - 'A']);	// Extended Response
						else
							printf(Mode_msg, eeprom.defval.mode[vfo_idx]);	// Standard Response
					break;

					case 'Q':						// Tuning rate, 0 = 1Hz, 1 = 10Hz, 2 = 100Hz
						printf(Tuning_Rate_msg, Tuning_Step);				// Send mode data
					break;

					case 'R':						// RIT Status & Frequency
						printf(Print_RIT, RIT_On_Off, rit);	// RIT On/Off 0 = Off, 1 = On
					break;

					case 'S':						// Current S-Meter or O/P Power Meter Reading
						if(PTT_IN == 0)						// TX on ?
							printf(Meter_msg, pwr_meter); 	// PWR meter
						else
							printf(Meter_msg, s_meter); 	// S-meter
					break;

					case 'T':						// Tx/Rx Status
						if(PTT_IN == 0)				// PTT_IN = negative logic, 0 = TX
							printf(Tx_msg);			// PTT status, 0 = Off, 1 = On
						else
							printf(Rx_msg);
					break;

					case 'U':						// Firmware Version
						printf(Firmware_Ver, VERSION);
					break;

					case 'V':						// Current VFO Status
						if(a == 'A' || a == 'B')	// Extended Command
							{
							b = a - 'A';
							printf(VFO_status_msg, a, eeprom.defval.vfo[b], eeprom.defval.mode[b], check_bit(b));
							}
						else
							printf(VFO_msg, Select_VFO);	// Send VFO number. 0 = VFO-A, 1 = VFO-B, 2 = Split Mode
					break;

					case 'W':						// Query Filter Bandwidth
						if(a == 'N' || a == 'M' || a == 'W')	// The 'M' option is to retain compatibility with F6DEX's control program
							{
							b = (a == 'M' || a == 'W');			// If true, b = 1, which is the new WIDE bandwidth.
							printf(Extended_BW_msg, a, (75000L / eeprom.defval.scaf_filters[b]));
							}
						else									// In this case, b = 0, the new NARROW bandwidth.
							printf(BW_msg, (75000L / eeprom.defval.scaf_filters[RX_Bandwidth]));

					break;

					case 'Y':						// Extended Display Status
						printf(Extended_Display, sub_page0);
					break;
					}
				}
			else if(cmd_buf[0] == '=')				// Command
				{
				switch(cmd_buf[1])
					{
					case 'A':						// Set filter, 0 = Narrow, 1 = Wide
						RX_Bandwidth = get_num(2, 2);
					break;

					case 'B':						// Set Band. B1 = 160m to B9 = 10m
						if(freq_change_flag)
							{
							b = get_num(2, 10);

							if(b < 1) b = 1;

							b--;

							if(RBS_Mode == _DEFAULT)
								eeprom.defval.vfo[vfo_idx] = amateur_bands[b];
							else
								eeprom.defval.vfo[vfo_idx] = eeprom.defval.vfo[b + 30];

							if(b > 2)
								eeprom.defval.mode[vfo_idx] = USB;
							else
								eeprom.defval.mode[vfo_idx] = LSB;
							}
					break;

					case 'C':						// Speech Processor (Compressor)
						Speech_Processor = get_num(2, 2);
					break;

					case 'E':						// Equalise, E0: VFOA = VFOB, E1: VFOB = VFOA
						if(freq_change_flag)
							{
							if(get_num(2, 2) == 1)
								{
								eeprom.defval.vfo[1] = eeprom.defval.vfo[0];
								eeprom.defval.mode[1] = eeprom.defval.mode[0];
								}
							else
								{
								eeprom.defval.vfo[0] = eeprom.defval.vfo[1];
								eeprom.defval.mode[0] = eeprom.defval.mode[1];
								}
							}
					break;

					case 'F':						// Set frequency
						if(freq_change_flag)
							{
							if(a == 'A' || a == 'B')				// Extended Command
								{
								b = a - 'A';
								i = 3;
								}
							else
								{
								b = vfo_idx;
								i = 2;
								}

							f = atol(cmd_buf + i);	// Get new frequency

							if(f > 30000000)		// Check limits
								f = 30000000;

							eeprom.defval.vfo[b] = f;				// Frequency is already in decimal units

							if(auto_inform && vfo_freq[b] != eeprom.defval.vfo[b])
								{
								vfo_freq[b] = eeprom.defval.vfo[b];

								if(i == 3)
									printf(VFO_status_msg, a, eeprom.defval.vfo[b], eeprom.defval.mode[b], check_bit(b));
								else
									printf(Set_Freq_Cmd, eeprom.defval.vfo[b]);
								}
							}
					break;

					case 'G':						// AGC Fast/Slow
						eeprom.defval.agc_speed = get_num(2, 2);
					break;

					case 'I':
						auto_inform = get_num(2, 4);		// Auto-Inform Enable/Disable

						if((auto_inform) && maximum_VFO != SPLIT_MODE)	// If A/B+Split mode is not already selected
							{
							maximum_VFO = SPLIT_MODE;		// Select Split Mode
							Select_VFO = 0;					// Set current VFO to A
							}

						latency_count = 50 * auto_inform;	// Set latency counter time, mSec

						if(auto_inform)						// Reset state values
							{
							reset_state();
							chk_state(1);
							}
					break;

					case 'J':								// Set CW Shift
						eeprom.defval.cw_shift = atoi(cmd_buf + 2);
						max_min(&eeprom.defval.cw_shift, MAX_CW_SHIFT, MIN_CW_SHIFT);
					break;

					case 'L':								// VFO Lock, 1 = Lock current VFO, 0 = Unlock current VFO
						if(a == 'A' || a == 'B')			// Extended Command
							{
							vfo_lock = get_num(3, 2);
							i = a - 'A';

							if(auto_inform && vfo_lock_status[i] != vfo_lock)
								{
								vfo_lock_status[i] = vfo_lock;
								printf(Extended_Lock_msg, a, vfo_lock);
								}
							}
						else
							{
							vfo_lock = get_num(2, 2);
							i = vfo_idx;
							}

						if(vfo_lock)
							set_bit(i);
						else
							clr_bit(i);
					break;

					case 'M':						// Set Mode. 0 = LSB, 1 = USB, 2 = CW, 3 = CWR, 4 = Tune
						if(freq_change_flag)
							{
							if(a == 'A' || a == 'B')		// Extended Command
								{
								b = a - 'A';				// Extract VFO, A or B
								i = get_num(3, 5);			// Extract desired mode, 0 - 4

								if(auto_inform && mode[b] != i)	// If we are using Auto-Inform, and the mode is different,
									{
									mode[b] = i;			// then set new mode,
									printf(Extended_Mode_msg, a, i);	// and report setting.
									}
								}
							else							// If we are not using Auto-Inform,
								{
								b = vfo_idx;				// get current VFO and
								i = get_num(2, 5);			// extract new mode.
								}

							eeprom.defval.mode[b] = i;		// Set VFO to new mode.
							}
					break;

					case 'P':		// Remote Power OFF
						clear_lcd();
						lcd_putst(Remote_Pwr_Off);
						stop_scaf_clock();

						if(get_num(2, 2) == 1)
							{
							lcd_cmd(LINE2);
							lcd_putst(Settings_Saved);
							save_defval();
							}

						ms_delay(1000);
						Switch_Off();						// No return from this function.
					break;

					case 'Q':								// Set tuning rate
						Tuning_Step = get_num(2, 3);
					break;

					case 'R':								// Set RIT 0 = Off, 1 = On
						RIT_On_Off = get_num(2, 2);
					break;

					case 'T':								// Tx/Rx
						PTT_OUT = get_num(2, 2);
					break;

					case 'V':								// Set VFO
						if(freq_change_flag)				// If a change is permissible
							{
							b = get_num(2, 3);

							if(b == 2 && maximum_VFO != SPLIT_MODE)	// If Split Mode not already selected
								maximum_VFO = SPLIT_MODE;	// Select Split Mode

							Select_VFO = b;
							}
					break;

					case 'W':								// Set selected filter bandwidth (=Wnnnn)
						if(a == 'N' || a == 'M' || a == 'W')	// Extended Command. The 'M' option is to retain compatibility with F6DEX control program.
							{
							b = (a == 'M' || a == 'W');		// 0 = Narrow, 1 = Wide

							set_filter_width(3, b);
							}
						else
							set_filter_width(2, RX_Bandwidth);

					break;

					case 'X':								// Exchange VFO-A & VFO-B frequency and mode
						f = eeprom.defval.vfo[0];			// Save VFO-A frequency
						b = eeprom.defval.mode[0];			// Save VFO-A mode
						eeprom.defval.vfo[0] = eeprom.defval.vfo[1];
						eeprom.defval.vfo[1] = f;
						eeprom.defval.mode[0] = eeprom.defval.mode[1];
						eeprom.defval.mode[1] = b;
					break;

					case 'Y':								// Extended Display
						sub_page0 = get_num(2, 5);
					break;
					}
				}
			clear_buffer();									// Clear the command buffer for the next round
			}
		else
			{
// Collect data
			if(cmd_buf_idx < MAX_TRX2_CMD)
				cmd_buf[cmd_buf_idx++] = c;					// Save data and increment index
			else
				clear_buffer();								// Unknown command, or buffer over-run.
			}
		}
	}

