// JUMA-TRX2 service & setup routines
// Juha Niinikoski, OH2NLT

// Service routines moved to separate file service.c 27.12.2006
// Some displays changed, TX can operate, last power off VFO & freq in use
// Minor text corrections 30.10.2008
// Revised display prompts - 5B4AIY
// Added adjustment limit checks - 5B4AIY
// Added Overcurrent, Over Voltage, and Under Voltage trips - 5B4AIY
// Moved Band Limit setting from User Configuration module to here - 5B4AIY
// Added High SWR Alarm adjustable delay. 5B4AIY

#include <stdio.h>							// Added - 5B4AIY
#include "juma-trx2.h"						// Project definitions, macros and constants
#include "trx2_eeprom.h"					// EEPROM structure definitions

#define MAX_SERVICE_PAGES	17				// Number of display pages

// External References
// External Functions
extern void beep(int, int);						// Generate beep, period, length
extern void ms_delay(unsigned int);				// Delay
extern int encoder_get(void);					// Read encoder
extern void reset_encoder(void);				// Reset encoder
extern void lcd_putst(register const char *);	// LCD put string
extern void disp_meter(double, int);			// Display meters
extern void draw_s_meter(int);					// Display scaled S-meter
extern void disp_fwd_pwr(void);					// Display watt-meter
extern void disp_id(void);						// Display RF Amp Drain current
extern void get_pwr(void);						// Get RF power data
extern void send_packet_to_74hc595(void);		// Main & PA board I/O
extern int convert_adc12(unsigned int);			// Added - 5B4AIY
extern int encoder_get_coarse(int);				// Added - 5B4AIY
extern int save_settings(int, int);				// Added - 5B4AIY
extern int read_defval(void);
extern void get_value(int *, int, int, int);	// Added - 5B4AIY
extern void set_decade_value(int *, int, int);	// Added - 5B4AIY
extern void set_chgen(int);						// Program custom LCD characters
extern void dds_load(unsigned long);			// Added - 5B4AIY
extern void max_min(int *, int, int);
extern void warning_beep(void);
extern void get_one_zero(int *);
extern void debounce(void);
extern void synch_beep();
extern long int_round(long, long);
extern void display_atten(int);
extern void display_line(int, const char *);
extern void increment_and_check(int *, int);
extern void wait_PWR_MODE_rls(void);
extern void test_button(void);

// External Data
extern int fwd_adval;				// Forward & reverse power raw values, measured only when TX
extern int rev_adval;
extern long fwd_pwr;				// Relative forward power
extern long rev_pwr;				// Relative reverse power
extern long out_pwr;				// Scaled output power = (fwd_pwr - rev_pwr) * scale
extern int id_raw_value;			// Raw drain current
extern int PTT_IN;					// PTT switch value
extern char lcdpbuff[];				// Standard LCD display buffer
extern char *band_limits[];
extern char *on_off[];
extern char spc_or_bar[];
extern char dp_4s_11s_fmt[];		// Standard format descriptor
extern char _mode[];
extern char _16s[];
extern int s_meter_const;			// Calculated S-Meter Calibration Constant
extern long freq_in_use;
extern int vfo_idx;
extern int latch;
extern int svc_flag;
extern long button_count;

// Local Data
const char cal_factor[] = {"Cal Factor: %4i"};
const char trip[] = {"Trip"};
const char trip_msg[] = {"Limit:%9.2f%c"};
const char *service_prompt[] = {						// LCD display prompt messages (Line 1)
								"Set Ref Osc Freq",		// 0
								"Supply:  ",			// 1
								"RF Amp:  ",			// 2
								"Forward: ",			// 3
								"",						// 4 - S-meter display is on the first line, this is a place-holder
								"Beep Time  0=Off",		// 5
								"High SWR Trip   ",		// 6
								"SWR Alarm Delay ",		// 7
								"Overcurrent     ",		// 8
								"Overcurrent Trip",		// 9
								"Overvoltage     ",		// 10
								"Overvoltage Trip",		// 11
								"Low Voltage     ",		// 12
								"Low Voltage Trip",		// 13
								"Second Function ",		// 14
								"RF Equalisation ",		// 15
								"Regional Setting",		// 16
								"Splash Screen   "		// 17
								};
const char *settings_saved[] = {
								"Existing Calibration settings re-loaded from",
								"New Calibration settings saved to"
								};
const char *button_type[] = {
								"Press&Hold",
								"Latched"
								};
const char *splash_type[] = {							// Boot-Up Splash Screen Message (Indexed by: cal.calval.splash)
							"Off",						// 0
							"Juma TRX2",				// 1
							"Juma+Region"				// 2
							};
const char svc_0_msg[] = {"Osc:%9li Hz"};
const char svc_4_msg[] = {"S-Meter"};
const char svc_5_msg1[] = {"Beep:%8i mS"};
const char svc_5_msg2[] = {"Beep:%11s"};
const char svc_6_msg[] = {"Limit:%8.1f:1"};
const char svc_7_msg1[] = {"Time:%6.1f Secs"};
const char svc_7_msg2[] = {"Disabled"};
const char Display_Next_msg[] = {"\n\rService & Calibration Mode\n\rDISPLAY/CONFIG Short Push: Parameter Select\n\rDISPLAY/CONFIG Long Push : Save & Exit\n\r"};
const char EEPROM_Saved[] = {"\n\r%s EEPROM.\n\r"};

int cal_page, no_adj = FALSE, rf_band = 8;	// rf_band + 3 = index for cal.calval.rf_atn_bits[], where 3 = 160m, to 11 = 10m, we start at 10m.

// User Configuration Data
extern union
	{
	struct defval defval;
	} eeprom;

// System Calibration Values
extern union
	{
	struct calval calval;
	} cal;

// Service & calibration functions
void display_alarm_msg(int flag)
	{
	sprintf(lcdpbuff, dp_4s_11s_fmt, trip, (flag) ? on_off[ON] : on_off[OFF]);
	}

long set_range(long value, int rate, long cal_factor, unsigned long min, unsigned long max)
	{
	if(no_adj == FALSE)		// If Fast Page Select is not active, adjust value, else skip
		value += (rate) ? encoder_get_coarse(rate) : encoder_get();

	value *= cal_factor;
	value = (value > max) ? max : (value < min) ? min : value;
	return value / cal_factor;
	}

void set_flag(int enable, int disable)
	{
	int i;

	if(no_adj == FALSE)						// If Fast Page Select in not active, ajust value, else skip.
		{
		i = encoder_get_coarse(ENCODER_SLOW);

		if(i > 0) cal.calval.alarm_flags |= enable; 	// Enable
		if(i < 0) cal.calval.alarm_flags &= disable;	// Disable
		}
	}

void inc_band(int *band)
	{
	(*band)--;
	if(*band < 0) *band = 8;

	if(*band == 8) beep(HZ466_85, 100);		// Signal 10m band, start position
	else beep(HZ587_31, SHORT_BEEP);
	}

void svc_0(void)							// Set Master Clock calibration factor
	{
	cal.calval.ref_osc = set_range(cal.calval.ref_osc, SLOW, 1, MIN_OSC, MAX_OSC);
	sprintf((char *)lcdpbuff, svc_0_msg, cal.calval.ref_osc);	// Added cast - 5B4AIY
	}

void svc_1(void)							// Set battery voltage calibration factor
	{
	double batt;

	batt = (double)convert_adc12(BATT_CH);
	batt *= (double)cal.calval.batt_mult;
	disp_meter(batt, VOLTS);										// New version 5B4AIY
	get_value(&cal.calval.batt_mult, ENCODER_NORMAL, BATT_MULT_MIN, BATT_MULT_MAX);
	sprintf((char *)lcdpbuff, cal_factor, cal.calval.batt_mult);	// Show scale multiplier - Added cast - 5B4AIY
	}

void svc_2(void)							// Set drain current calibration factor
	{
	id_raw_value = convert_adc12(ID_CUR);
	disp_id();														// Display drain current
	get_value(&cal.calval.id_mult, ENCODER_NORMAL, ID_MULT_MIN, ID_MULT_MAX);
	sprintf((char *)lcdpbuff, cal_factor, cal.calval.id_mult);		// Added cast - 5B4AIY
	}

void svc_3(void)							// Set power meter calibration factor
	{
	get_pwr();
	disp_fwd_pwr();													// Display Watt-meter, power is displayed in Watts
	get_value(&cal.calval.fwd_pwr_mult, ENCODER_NORMAL, FWD_PWR_MULT_MIN, FWD_PWR_MULT_MAX);
	sprintf((char *)lcdpbuff, cal_factor, cal.calval.fwd_pwr_mult);	// Added cast - 5B4AIY
	}
/*
  The S-meter is designed with a DC output voltage of 1V for S9, and 2V for S9+40dB. A 1V input to the A-D
  converter results in a count of 820. Therefore the scaling factor to read half-scale is 820 / 24 = 34.
  The scale is a 48-point scale, and for a 2V input with a count of 1640 and a scale factor of 34 results
  in a value of 48. The scaling factors are therefore arranged from 5 to 70 to take into account the worst
  possible scaling factors required. If a later main board is used with a substitute transistor for T4, then
  the default scaling factor is about 17 as this transistor's threshold voltage is higher, and thus the AGC
  voltage is lower for S9 and S9+40dB.
*/
void svc_4(void)							// Set S-meter calibration factor
	{
	draw_s_meter((int)int_round((long)convert_adc12(S_METER), (long)cal.calval.s_mtr_cal));	// Display actual meter, using the graphic meter scale
	lcd_putst(svc_4_msg);
	get_value(&cal.calval.s_mtr_cal, ENCODER_SLOW, S_METER_CAL_MIN, S_METER_CAL_MAX);
	sprintf((char *)lcdpbuff, cal_factor, cal.calval.s_mtr_cal);	// Added cast - 5B4AIY
	}

void svc_5(void)							// Set annunciator tone length
	{
	get_value(&cal.calval.beep_len, ENCODER_NORMAL, BEEP_LEN_MIN, BEEP_LEN_MAX);

	if(cal.calval.beep_len)
		sprintf((char *)lcdpbuff, svc_5_msg1, cal.calval.beep_len);	// Show current setup - Added cast - 5B4AIY
	else
		sprintf((char *)lcdpbuff, svc_5_msg2, on_off[OFF]);
	}

void svc_6(void)							// Set SWR trip level
	{
	set_decade_value(&cal.calval.swr_trip, MIN_SWR_ALARM, MAX_SWR_ALARM);
	sprintf((char *)lcdpbuff, svc_6_msg, ((double)cal.calval.swr_trip) / 100.0);	// Show current setup - Added cast - 5B4AIY
	}

void svc_7(void)							// Adjust SWR Alarm Delay time
	{
	get_value(&cal.calval.swr_delay, ENCODER_SLOW, MIN_SWR_DLY, MAX_SWR_DLY);

	if(cal.calval.swr_delay)
		sprintf((char *)lcdpbuff, svc_7_msg1, (double)cal.calval.swr_delay);
	else
		sprintf((char *)lcdpbuff, _16s, svc_7_msg2);
	}

void svc_8(void)							// Enable/Disable the Over-Current Alarm
	{
	set_flag(OC_AL_ON, OC_AL_OFF);
	display_alarm_msg(cal.calval.alarm_flags & OC_AL_ON);
	}

void svc_9(void)							// Set Over-Current trip point
	{
	if(cal.calval.alarm_flags & OC_AL_ON)	// If Over-Current Alarm is enabled,
		{
		cal.calval.overcurrent_trip = (int)set_range((long)cal.calval.overcurrent_trip, SLOW, (long)cal.calval.id_mult, (OVERCURRENT_LOWER * 2000UL), (OVERCURRENT_UPPER * 2000UL));
		sprintf((char *)lcdpbuff, trip_msg, ((double)cal.calval.overcurrent_trip * (double)cal.calval.id_mult) / 2000000.0, 'A');	// Show current setup - Added cast - 5B4AIY
		}
	else									// Else, skip this page
		cal_page++;
	}

void svc_10(void)							// Enable/Disable the Over-Voltage Alarm
	{
	set_flag(HI_V_AL_ON, HI_V_AL_OFF);
	display_alarm_msg(cal.calval.alarm_flags & HI_V_AL_ON);
	}

void svc_11(void)							// Set Over-Voltage trip point
	{
	if(cal.calval.alarm_flags & HI_V_AL_ON)	// If Over-Voltage Alarm is enabled...
		{
		cal.calval.overvoltage_trip = (int)set_range((long)cal.calval.overvoltage_trip, ENCODER_NORMAL, (long)cal.calval.batt_mult, (OVERVOLTAGE_LOWER * 10000UL), (OVERVOLTAGE_UPPER * 10000UL));
		sprintf((char *)lcdpbuff, trip_msg, ((double)cal.calval.overvoltage_trip * (double)cal.calval.batt_mult) / 1000000.0, 'V');		// Show current setup - Added cast - 5B4AIY
		}
	else									// Else, skip this page
		cal_page++;
	}

void svc_12(void)							// Enable/Disable Under-Voltage Alarm
	{
	set_flag(LO_V_AL_ON, LO_V_AL_OFF);
	display_alarm_msg(cal.calval.alarm_flags & LO_V_AL_ON);
	}

void svc_13(void)							// Set Under-Voltage trip point
	{
	if(cal.calval.alarm_flags & LO_V_AL_ON)	// If Under-Voltage Alarm is enabled...
		{
		cal.calval.undervoltage_trip = (int)set_range((long)cal.calval.undervoltage_trip, ENCODER_NORMAL, (long)cal.calval.batt_mult, (UNDERVOLTAGE_LOWER * 10000UL), (UNDERVOLTAGE_UPPER * 10000UL));
		sprintf((char *)lcdpbuff, trip_msg, ((double)cal.calval.undervoltage_trip * (double)cal.calval.batt_mult) / 1000000.0, 'V');		// Show current setup - Added cast - 5B4AIY
		}
	else									// Else, skip this page
		cal_page++;
	}

void svc_14(void)							// Select Button Second Function mode, Press & Hold or Latched
	{
	get_one_zero(&hold_latch);
	sprintf(lcdpbuff, dp_4s_11s_fmt, _mode, button_type[hold_latch]);
	}

void svc_15(void)							// Adjust individual band RF attenuator settings
	{
	static int rpt;
	int atten;

	if(_FILTER && !no_adj)					// If FILTER button is pressed, and adjustment is allowed,
		{
		if(rpt)								// and repeat timer is TRUE,
			{
			rpt++;							// then increment the repeat timer,

			if(rpt > 6)						// and if it has timed out,
				{
				inc_band(&rf_band);			// increment the band number,
				rpt = 2;					// and reset the timer in anticipation of another increment.
				}
			}
		else								// If repeat timer is FALSE,
			{
			inc_band(&rf_band);				// then this is the first time, so increment the band number,
			rpt = 1;						// and start the repeat timer.
			}
		ms_delay(100);
		}

	if(SW6 && rpt)							// If FILTER button is released, and the repeat timer is TRUE,
		{
		rpt = 0;							// then reset the repeat timer,
		debounce();							// and de-bounce the button release.
		}

	atten = cal.calval.rf_atn_bits[rf_band + 3];
	get_value(&atten, ENCODER_SLOW, MIN_ATTEN, MAX_ATTEN);
	cal.calval.rf_atn_bits[rf_band + 3] = atten;
	display_atten(rf_band << 8);
	}

void svc_16(void)							// Select IARU/USA (Extra)/USA (General) Band Limts
	{
	get_value(&cal.calval.Band_Limits, ENCODER_SLOW, IARU_1, USA_GENERAL);
	sprintf(lcdpbuff, _16s, band_limits[cal.calval.Band_Limits]);
	}

void svc_17(void)							// Splash Screen display On/Off
	{
	get_value(&cal.calval.splash, ENCODER_SLOW, OFF, JUMA_REGION);
	sprintf(lcdpbuff, dp_4s_11s_fmt, _mode, splash_type[cal.calval.splash]);
	}

void (*set_svc[])(void) = {			// Indexed by cal_page
							svc_0,	// 0	Set Master Clock calibration factor
							svc_1,	// 1	Set battery voltage calibration factor
							svc_2,	// 2	Set drain current calibration factor
							svc_3,	// 3	Set power meter calibration factor
							svc_4,	// 4	Set S-meter calibration factor
							svc_5,	// 5	Set annunciator tone length
							svc_6,	// 6	Set SWR trip level
							svc_7,	// 7	Adjust SWR Alarm Delay time
							svc_8,	// 8	Enable/Disable the Over-Current Alarm
							svc_9,	// 9	Set Over-Current Alarm threshold if enabled
							svc_10,	// 10	Enable/Disable the Over-Voltage Alarm
							svc_11,	// 11	Set Over-Voltage Alarm Threshold if enabled
							svc_12,	// 12	Enable/Disable Under-Voltage Alarm
							svc_13,	// 13	Set Under-Voltage Alarm Threshold if enabled
							svc_14,	// 14	Set button Second Function mode, Hold/Latch
							svc_15,	// 15	Set equalisation attenuation settings
							svc_16,	// 16	Select IARU/USA (Extra)/USA (General) Band Limits
							svc_17	// 17	Splash Screen display type (0 = Off, 1 = Juma TRX2, 2 = Juma+Region)
							};

void display_svc_page(void)
	{
	display_line(LINE1, service_prompt[cal_page]);	// Display appropriate page prompt message,
	set_svc[cal_page]();							// then execute the adjustment function,
	display_line(LINE2, lcdpbuff);					// and finally display the return message.
	}

void skip_page(int cal_page, int *page)
	{
	if(cal_page <= *page) (*page)++;
	else (*page)--;
	}

void fast_svc_page(void)
	{
	int page;

	page = cal_page;
	page += encoder_get_coarse(ENCODER_MEDIUM);
	max_min(&page, MAX_SERVICE_PAGES, 0);

	if(page == 9 && !(cal.calval.alarm_flags & OC_AL_ON))		// If Over-Current Flag is reset,
		skip_page(cal_page, &page);								// then skip page.

	if(page == 11 && !(cal.calval.alarm_flags & HI_V_AL_ON))	// If Over-Voltage Flag is reset,
		skip_page(cal_page, &page);								// then skip page.

	if(page == 13 && !(cal.calval.alarm_flags & LO_V_AL_ON))	// If Under-Voltage Flag is reset,
		skip_page(cal_page, &page);								// then skip page.

	cal_page = page;
	no_adj = TRUE;
	display_svc_page();
	no_adj = FALSE;
	}

void service(void)
	{
	eeprom.defval.Tune_Attenuation = 3;				// Set Tune Attenuation to 3dB.
	AF_Input = LINE;								// Force audio input to LINE for maximum attenuation.
	eeprom.defval.vfo_sel = VFO_A;					// Use VFO-A,
	eeprom.defval.vfo[VFO_A] = SERVICE_FREQ;		// and set it to 14.019300MHz.
	eeprom.defval.Power_Meter = FALSE;				// Force power meter to display Watts.
	vfo_idx = VFO_A;								// Use VFO-A,
	RX_Bandwidth = WIDE;							// and set the bandwidth to wide.
	freq_in_use = SERVICE_FREQ;						// Set 14.019300MHz as well,
	dds_load(freq_in_use);							// and load synthesiser.
	set_chgen(0);									// Use original S-Meter scale
	cal_page = 0;									// Start at the first page

	if(Serial_Mode == SERIAL_TEST)					// If the serial port protocol is set to TEST
		printf(Display_Next_msg);

	do	{
		button_count = DEBOUNCE_COUNT;				// Set each time around loop, so that it is ready for de-bounce count.
		no_adj = FALSE;
		eeprom.defval.mode[VFO_A] = (cal_page == 3) ? TUNE : USB;
		display_svc_page();
// PTT 
		if(PTT_IN_IO)			// Set PTT status 1 = Receive, 0 = Transmit
			{
			PTT_IN = RECEIVE;	// Copy I/O bit to status flag
			KEY = OFF;			// Remove carrier
			}
		else
			{
			PTT_IN = TRANSMIT;
			KEY = ON;			// Carrier on
			}
// Main & PA board SPI bus
		send_packet_to_74hc595();

// Read switches
		if(PTT_IN)						// If in RECEIVE mode,
			{
			if(_MODE)					// Fast Calibration Page Select
				{
				button_debounce(_MODE);				// Wait for button to settle

				if(!button_count)		// ****** SECONDARY Function ******
					{					// There is no short-push primary function.
					reset_encoder();				// This fixes the spurious Fast Page Switch bug! Finally!

					if(hold_latch == LATCH)			// Long Push = Latched Mode
						{
						latch = TRUE;
						while(_MODE) warning_beep();// Warning beep while MODE button is still pressed
						synch_beep();

						do	{
							warning_beep();
							fast_svc_page();

							if(PWR_SW || _MODE)		// Exit latched mode if either the PWR or the MODE button is pressed.
								{
								latch = FALSE;		// Release the page hold latch
								beep(HZ587_31, cal.calval.beep_len);
								}
							} while(latch);
						}
					else							// Long Push = Press & Hold Mode
						{
						beep(HZ466_85, cal.calval.beep_len * 10);	// Long Beep for Fast Calibration Page Switch
						while(_MODE) fast_svc_page();// Fast Service Mode page switch displayed as long as MODE is pressed
						}
					}
				wait_PWR_MODE_rls();
				}

			if(_DISPLAY)					// DISPLAY/CONFIG - Select calibration function page
				{
				int msg;							// Local Variable

				button_debounce(_DISPLAY);

				if(button_count)		// ****** PRIMARY Function ******
					{								// Change Calibration Page
					increment_and_check(&cal_page, MAX_SERVICE_PAGES + 1);
					beep((cal_page) ? HZ587_31 : HZ466_85, SHORT_BEEP);
					}
				else					// ****** SECONDARY Function ******
					{								// Exit Service Mode
					beep(HZ466_85, SHORT_BEEP);		// Short beep for return to normal operation
					msg = save_settings(0, 1);		// Save new calibration settings

					if(Serial_Mode == SERIAL_TEST)	// If serial port mode is TEST, print settings message
						printf(EEPROM_Saved, settings_saved[msg]);

					read_defval();					// Restore the original configuration settings, notably VFO frequency and mode,
					reset_encoder();				// clear encoder to prevent spurious frequency changes,
					svc_flag = FALSE;				// and exit service mode.
					latch = FALSE;
					wait_PWR_MODE_rls();
					}
				while(_DISPLAY);					// Wait until button released
				debounce();
				}
			}
		} while(svc_flag);	// End WHILE loop
	}	// End Function

