// dsPIC30 Timers PWM DAC for JUMA-TRX2- 5B4AIY - 30/NOV/2012
// Juha Niinikoski, OH2NLT 30.11.2006

// Microchip C30 compiler conversion

/*
 JUMA-TRX2 Timer & PWM system

 Timer & PWM unit usage
 TMR1 is used for tone generation, IRQ driven system, RD1/OC2 = Tone O/P
 TMR2 is uded for SCAF clock generation
 OC1 output is for SCAF filter clock
 OC1 is connected to TMR2
 TMR3 is used for 1ms timer tick IRQ and for LCD PWM dac timebase
 OC3 output is for LCD back light control
 OC4 output is for LCD contrast
 OC3 and OC4 are connected to TMR3

 TMR4 reserved for CW timing
 TMR5 used for delays (separate module)
 Clickless tone generator code 16.12.2006
 Tune mode start glitch removed 15.01.2007
 1 second counter added for ISD1700 routines
 Tone generator end click problem corrected 17.03.2007
 Yaesu CAT protocol time-out counter 22.01.2008
 Adjustable CW side-tone 02.01.2008
 LCD back light timer added 03.07.2008
 Replaced functions: start_scaf_clock(), stop_scaf_clock(), set_pwm3_dac(), set_pwm4_dac(), and tone_off()
 with equivalent macro definitions in: juma-trx2.h. A.Ryan
 
 When using the CW Straight mode and connecting a real Vibroplex key, because of contact bounce on the initial contact closure, this leads
 to 'scratchy dots'. An examination of the keying waveform shows significant contact bounce for the first 5 to 10 mSec. By introducing another
 local variable - cb_suppression - this contact bounce can be suppressed. cb_suppression is initially set to zero. At the first contact closure the
 output is keyed, and cb_suppression is set to the value of CB_SUPPRESSION_TIME defined in juma-trx2.h. As a result, all contact closures and opens
 are now ignored until cb_suppression has decremented to zero in the 1mS interrupt timer. This effectively removes contact bounce without introducing
 any delays, and without affecting the maximum keying speed. A.Ryan - 5B4AIY - 27/DEC/2017
*/
#define TICK_PERIOD		7500		// 1ms tick
#define SCAF_05 		150			// 500Hz / 50kHz clock
#define SCAF_19 		39			// 1,9kHz / 192,307kHz clock
#define SCAF_24 		31			// 2,4kHz / 241,9kHz clock
#define SCAF_25 		30			// 2,5kHz / 250kHz clock

#include "juma-trx2.h"
#include "trx2_eeprom.h"			// Get EEPROM structure definitions

// External Variables
extern int cw_active;				// CW active flag for keyer code
extern int keyer_mode;				// 0 = Dot priority, 1 = Iambic A, 2 = Iambic B, 3 = Straight, -1 = Startup inhibit
extern int seconds;					// Free running seconds counter for Voice Memory module
extern int VM_Mode;					// Voice Memory Mode flag. -1 = Record, +1 = Play, 0 = Idle
extern int sq_timer;				// Squelch timer
extern int bl_timer;				// LCD back light timer
extern int tx_enable;				// Declared in juma-trx2.c - 5B4AIY True if in-band or if TX Disable is OFF
extern int s_meter_timer;
extern int loop_ctr;				// Alarm blink timer
extern int alarm_flag;

// Local Variables
unsigned int ms_counter;			// General mod 65535 ms counter
unsigned int tone_counter; 			// Beep tone length (ms)
static unsigned int busy_counter;	// Tone generator busy(ms), busy if != 0
static int key_state;				// Key state flag for straight mode tone generator
static int last_cycle;				// For click-less end trick
static int iambic_flag;				// Last played, 0 = Dot, 1 = Dash
static int dot;						// DOT RS flip flop, follows paddle logic, 0 = Closed
static int dash;					// DASH RS flip flop, follows paddle logic, 0 = Closed
static int cb_suppression = FALSE;	// Contact bounce suppression counter
int cw_break_timer;					// CW break timer, sets the CW transmit to receive delay time.
int cw_period;						// CW period(ms) (Dot time)
int cmd_timeout;					// RS232 command time-out, 0 = Time-out, Negative values = State Flags.
unsigned int pa100_tx_tmr;			// TRX2 protocol band data TX rate
int sample_flag;					// Sample flag for Juma TRX-2/PA-100D protocol, see serial_trx2.c
int decay_counter = 0;				// Digital power meter fast decay timer.
int swr_al_dly;						// SWR alarm activation delay counter.

// User Configuration Data
extern union
	{
	struct defval defval;
	} eeprom;

// Initialise timers & PWM system
void init_timers_pwm(void)
	{
	dot = 1;						// Reset paddle flip flops
	dash = 1;

	ms_counter = MS_TIMER;			// Initialise the mS counter
	OC3CON = 0;						// First turn off the module if it was on
	OC4CON = 0;

// LCD PWM DAC setup
	OC3RS = DEFAULT_BL;				// Some small value for back light
	OC3R = DEFAULT_BL;
	OC3CON = 0x000E;				// Use TMR3, OC3 = PWM mode

	OC4RS = DEFAULT_CONTRAST;		// Something for contrast to make display visible
	OC4R = DEFAULT_CONTRAST;
	OC4CON = 0x000E;				// Use TMR3, OC4 = PWM mode

	PR3 = TICK_PERIOD - 1;			// Set PWM cycle, TMR3 period register, 1000Hz/1ms
	T3CONbits.TON = 1;				// Start TMR3

	_T3IE = 1;						// Enable TMR3 tick IRQ
	}

void init_scaf_clock(void)
	{
// SCAF clock
	OC1CON = 0;						// First turn off the module if it was on
	OC1RS = SCAF_25 / 2;			// Set OC1 duty cycle to 50%
	OC1R = SCAF_25 / 2;
	OC1CON = 0x0006;				// Use TMR2, OC1 = PWM mode
	PR2 = SCAF_25;					// SCAF clock frequency, wide filter
	T2CONbits.TON = 1;				// Start TMR2
	}

// PWM set functions
void set_scaf_clock(unsigned int scaf)	// SCAF clock
	{
	if(scaf < 2) scaf = 2;			// Protect PR2 setup

	OC1RS = scaf >> 1;				// Try to keep 50% duty cycle
	PR2 = scaf - 1;
	}

// Tone generator
void __attribute__((interrupt)) _T1Interrupt(void)
	{
//	_T1IF = 0;
	switch(last_cycle)				// Tone generator state machine
		{
		case 0: default:			// Tone output off, tri-state
			TONE_TRIS = 1;			// Tristate output
			_T1IE = 0;				// Disable TMR1 IRQs, Tone generator OFF
		break;

		case 1:						// Last half cycle
			TONE_OUT = !TONE_OUT;
			PR1 = PR1 >> 1;			// Next = half cycle
			last_cycle = 0;			// Flag for shut down
		break;

		case 2:						// Last full cycle, start shut down
			TONE_OUT = !TONE_OUT;	// Inverse tone out on every cycle

			if(TONE_OUT != 0)		// End with down going half cycle
				last_cycle = 1;		// Next = last full cycle
		break;

		case 3:						// Generate sound
			TONE_OUT = !TONE_OUT;	// Inverse tone out on every cycle
		break;
		}

	_T1IF = 0;
	}

// Tone set functions
void tone_on(int tone)				// Set tone, set value = 7,5MHz / tone(Hz) * 2
	{
	last_cycle = 3;					// Not last cycle, run state
	PR1 = tone - 1;					// Set tone period
	TMR1 = PR1 >> 1;				// Start with half period, reduce click
	TONE_OUT = 0;					// Start always same phase
	TONE_TRIS = 0;					// Enable output
	T1CONbits.TON = 1;				// Start timer
	_T1IF = 0;						// Clear old pending interrupt, start with clean half cycle
	_T1IE = 1;						// Enable TMR1 IRQs, Tone generator on
	}

void beep(int tone, int duration)
	{
	if(!busy_counter && duration && !key_state)	// Start tone only if tone generator free & duration specified
		{
		start_scaf_clock();			// Open audio path
		sq_timer = SQ_HOLD;			// Keep squelch open
		tone_counter = duration;	// Set tone length
		tone_on(tone);				// Set tone pitch & start play
		}
	}

void _cw_active(int multiplier)
	{
	PTT_OUT = 1;							// RF output on if CW mode
// cw_period is dot time in Iambic/Vibroplex mode, or user settable time in Straight key mode.
// multiplier is 7 in Iambic/Vibroplex mode, or 1 in Straight key mode.
	cw_break_timer = multiplier * cw_period;// TX to RX delay time.
	KEY = tx_enable;						// CW modulator on if tx_enable is true
	}

void key_down(int time)
	{
	if(!key_state)						// Do this only if the key was up
		{
		key_state = KEY_DOWN;					// Set key state flag,
		tone_on(eeprom.defval.cw_sidetone);		// and start the side-tone generator,
		cb_suppression = CB_SUPPRESSION_TIME;	// and set the contact bounce suppression timer.
		}

	if(cw_active) _cw_active(time);		// TX only if CW mode selected, CW release delay is set to 7 * dot time
	}

void key_up(void)
	{
	if(key_state)						// Do this only if the key was down
		{
		KEY = OFF;						// Turn the CW modulator off,
		tone_off();						// and stop side tone generator,
		key_state = KEY_UP;				// and reset the key state flag.
		}
	}

// Keyer code
void play_dot(void)						// Turn TX on & generate dot
	{
	iambic_flag = 0;					// Remember last played

	beep(eeprom.defval.cw_sidetone, cw_period);		// Start tone
	busy_counter = 2 * cw_period;		// Set busy period, 1 x dot time + 1 dot time

	if(cw_active) _cw_active(7);
	}

void play_dash(void)
	{
	iambic_flag = 1;					// Remember last played

	beep(eeprom.defval.cw_sidetone, (3 * cw_period));	// Start tone
	busy_counter = 4 * cw_period;		// Set busy period, 3 x dot time (dash) + 1 dot time

	if(cw_active) _cw_active(7);
	}

void _play_dot(void)
	{
	if(!dot && !busy_counter) play_dot();	// Start only if side-tone is not already running
	}

void _play_dash(void)
	{
	if(!dash && !busy_counter) play_dash();	// Start only if side-tone is not already running
	}

// IRQ code
// 1ms Tick timer
void __attribute__((interrupt)) _T3Interrupt(void)
	{
	_T3IF = 0;
/*
 Read paddle switches, set Dot/Dash RS flip-flops
 I/O Port DASH, 1 = Open, 0 = Closed
 I/O Port DOT,  1 = Open, 0 = Closed
 By assigning the actual logic values at the I/O port to local variables, 18 bytes are saved for each usage.
*/
	if(keyer_mode >= 0)					// Don't do anything during startup until the
		{								// keyer mode has been read from EEPROM. (Jan Wieck)
		if(keyer_mode == IAMBIC_B)		// Iambic B Mode
			{
			if(!DOT) dot = 0;			// Set RS flip-flop

			if(!DASH) dash = 0;
			}
		else							// All other Modes
			{
			dot = DOT;
			dash = DASH;
			}

		if(keyer_mode == VIBROPLEX && dot == dash)	// Special Vibroplex Mode
			{
			dot = dash = 1;
			}

// Run keyer logic
		switch(keyer_mode)					// Select keyer operation
			{
			case DOT_PRIORITY:
				if(!dot && !busy_counter) play_dot();// Start only if side-tone is not already running
				else if(!dash && !busy_counter) play_dash();
			break;

			case IAMBIC_A:
			case IAMBIC_B:
				if(iambic_flag)						// Last played was Dash, test Dot contact first
					{
					_play_dot();
					_play_dash();
					}
				else								// Last played was Dot, test Dash contact first
					{
					_play_dash();
					_play_dot();
					}
			break;

			case VIBROPLEX:
				if(!dot)
					{
					_play_dot();
					break;		// This is essential to prevent spurious operation with a dual-paddle key when both paddles are squeezed.
					}

				if(!dash) key_down(7);				// Key down
				else key_up();						// Key up
			break;

			case STRAIGHT_KEY:						// Straight Mode - Suppress contact bounce if a real Vibroplex key is connected
				if(cb_suppression) cb_suppression--;// Decrement the contact bounce suppression timer, and ignore any contact state changes.
				else								// Take note of contact state change.
					{
					if(!dot) key_down(1);			// Contact closure detected, key down, I/O bit, set key_state, and cb_suppression time.
					else key_up();					// Contact open detected, so do the key up logic, reset key_state.
					}
			break;
			}
		}		// End of (keyer_mode >= 0)

// General timing
	if(tone_counter)
		{
		tone_counter--;

		if(!tone_counter)
			{
			tone_off();					// Run stop with half cycle logic
//			TONE_TRIS = 1;				// Tristate output
//			_T1IE = 0;					// Disable TMR1 IRQs, Tone generator off
			if(eeprom.defval.mode[eeprom.defval.vfo_sel] != TUNE)	// If tune MODE selected, do not touch KEY bit
				KEY = OFF;				// Turn CW modulator off if any other MODE
			}
		}

	if(cw_break_timer)					// Run CW break timer
		{
		cw_break_timer--;

		if(!cw_break_timer) PTT_OUT = 0;// RF output OFF if CW mode selected
		}

	if(busy_counter)					// Decrement CW tone busy timer if active
		{
		busy_counter--;

		if(!busy_counter)				// Mark ready
			{
			if(!iambic_flag) dot = 1;	// Was Dot, so, reset Dot
			else dash = 1;				// Was Dash, so, reset Dash
			}
		}

	if(ms_counter) ms_counter--;					// Decrement counter. After 999 mS it will be zero, but,
	else											// this section will only execute on the next (1000th) interrupt.
		{
		ms_counter = MS_TIMER;						// Reset counter
		seconds += VM_Mode;

		if(bl_timer)								// Run LCD back light timer, if enabled,
			{
			bl_timer--;

			if(!bl_timer) bl_off();					// and if the timer has expired, turn the backlight off.
			}
		}
// Simple timers take 3 instructions, and thus add about 0.4uS to the loop execution time.
// Squelch timer
	if(sq_timer) sq_timer--;						// Run squelch timer

// RS232 time-out timer for Yaesu CAT protocol
	if(cmd_timeout > 0) cmd_timeout--;				// Negative values used as state flags

// TRX2 protocol timer - Used to limit band info TX rate to the PA100
	if(pa100_tx_tmr) pa100_tx_tmr--;				// See file: serial_trx2.c

// S-Meter hold timer
	if(s_meter_timer) s_meter_timer--;

// Alarm blink timer
	if(loop_ctr) loop_ctr--;

// TRX-2 Sample Flag
	if(sample_flag) sample_flag--;					// Sample Flag is reset in serial_trx2.c

// Digital power meter fast decay and warning beep timer
	if(decay_counter) decay_counter--;

// SWR Alarm Delay Timer
	if(swr_al_dly) swr_al_dly--;
	}

