// Simple UART interface for dsPIC30F6012
// Juha Niinikoski, OH2NLT 18.05.2005
/*
 Below is the code to initialize the USART
 8 bits, 1 start and 1 stop, no error handling
 kbhit and getch added 22.05.2005
 getche aded 02.06.2005 unsigned char
 Overrun error handling added 29.08.2005
 MPLAB C30 conversion, OH2GWE 2006.10.17
 Baud Rate setup & IRQ receiver for fast baud rates 14.01.2008
 Compiler bug work arround 01.02.2008
 UART1 init sequence changed to be compatible with later silicon 30.10.2008
 Data type cast corrections 18.11.2008

 Character To Integer Conversions (MicroChip C Compiler v2.056)
 This is the actual conversion results from the conversions as printed out.
 This has been tested with CI-Commander using the Yaesu CAT emulation.

 c = 0xff;
 uc = 0xff;

 Straight Assignment       : C =  00ff  I =  00ff	i = c;
 C cast to Integer         : C =  00ff  I =  00ff	i = (int)c;
 C cast to Unsigned Integer: C =  00ff  I =  00ff	i = (unisgned int)c;
 C masked to 0xFF assigned : C =  00ff  I =  00ff	i = c & 0x00ff;
 C masked 0x00ff and cast  : C =  00ff  I =  00ff	i = int(c & 0x00ff);
 Unsigned Char to Unsigned Int     :  00ff =  00ff	ui = uc;
 Unsigned Char cast to Unsigned Int:  00ff =  00ff	ui = (unsigned int)uc;
 A. Ryan - 5B4AIY - 3/APR/2014
*/

#include "juma-trx2.h"
#include <stdio.h>

//#define BAUD 9600
//#define FCY  1000000	// 4MHz XTAL
//#define FCY  8000000	// 4MHz XTAL x8 PLL = 32MHz
//#define FCY  16000000	// 4MHz XTAL x16 PLL = 64MHz
//#define FCY  29491200	// 7,3728MHz XTAL x16 PLL = 117,9648MHz

// Baud Rate divisors for 30 MHz osc =  7,5 MHz FCY 
/* Actual baud rate divisors
[0] 1200	389.625
[1] 2400	194.3125
[2] 4800	96.65625
[3] 9600	47.828125
[4] 19200	23.4140625
[5] 38400	11.20703125
[6] 57600	7.138020833
[7] 115200	3.069010417
*/

// Module variables & definitions
static volatile unsigned char rx_buffer[256];		// UART mod(256) RX queue, must be exactly 256 long
static volatile unsigned char rx_buf_out_idx = 0;	// Buffer output index, pointer behind in idx
static volatile unsigned char rx_buf_in_idx = 1;	// Buffer input index, pointer always "leading"

const unsigned int baud_rates[] = {
									390,			// 1200
									194,			// 2400
									97,				// 4800
									48,				// 9600
									23,				// 19,200
									11,				// 38,400
									7,				// 57,600
									3				// 115,200
									};

// UART1 IRQ service
// Queue character
void __attribute__((interrupt)) _U1RXInterrupt(void)
	{
  	unsigned char rx;

  	rx = U1RXREG;                        	// Data read

  	if((unsigned char)(rx_buf_in_idx + 1) != rx_buf_out_idx) 		// Check overrun
  		{
 		rx_buffer[rx_buf_in_idx] = rx;		// Place just received character in queue
		rx_buf_in_idx++;
    	}
// Else reject character

// Check & handle possible errors
	if(U1STAbits.OERR == 1)					// OERR is blocking UART, should not happen but..
		U1STAbits.OERR = 0;					// Reset overrun error if occured

// This is not automatic in 30F6014 UART
	IFS0bits.U1RXIF = 0;					// Reset IRQ flag
	}

// Clear serial port, reset input buffer
void ClearUSART1queue(void)
	{
	rx_buf_out_idx = 0;  					// Buffer output index
	rx_buf_in_idx = 1;   					// Buffer input index
	}

// UART1 init
void InitUSART1(void)
{
//U1BRG = ((FCY/16)/BAUD) - 1;   			// Set baud rate
	U1BRG = baud_rates[3];					// Set intial baud rate to 9600
	U1MODE = 0x0000;
	U1STA = 0x0000;							// IRQ on every character

// U1STAbits.UTXEN = 1;						// Enable transmitter
	IFS0bits.U1RXIF = 0;					// Clear possible pending IRQ
	IEC0bits.U1RXIE = 1;					// Enable RX IRQs
	U1MODEbits.UARTEN = 1;					// Enable UART
	U1STAbits.UTXEN = 1;					// Enable transmitter, must be after UARTEN for later silicon
	ClearUSART1queue();						// and ensure buffer pointers are reset.
}

// Set Baud Rate
void SetUSART1baud(int baud_sel)
	{
	 U1BRG = baud_rates[baud_sel];			// Set Baud rate
	}

// Print character
void putch(unsigned char c)
	{
	while(U1STAbits.UTXBF == 1);			// Wait here if TX FIFO full
	U1TXREG = (unsigned int)(c);			// Tx reg is 16-bits long
	}

// Check if key pressed, data available from receive buffer
// Not pressed = 0, Pressed = number of characters
// IRQ based RX queue check
int kbhit(void)
	{
	return (rx_buf_in_idx - (unsigned char)(rx_buf_out_idx + 1)); // Return with number of available characters
	}

// Get character from RX queue, IRQ RX system
unsigned char getch(void)
	{										// The compiler produces exactly the same code for: while(kbhit() == 0);
 	while (!kbhit());						// Wait for data if queue is empty 
  	return rx_buffer[++rx_buf_out_idx];		// Return with data
	}

// Receive character with echo
unsigned char getche(void)
	{
	unsigned char c;

	putch(c = getch());
	return c;
	}

